Object subclass:#HTMLDocumentPrinter
        instanceVariableNames:'style directory bindings'
        classVariableNames:''
        poolDictionaries:''
        category:'System-Documentation'
!

!HTMLDocumentPrinter  class methodsFor:'documentation'!

examples

"
    example 1: document
    ----------

        HTMLDocumentPrinter print:'/home2/stx/lib/doc/online/english/TOP.html'

    example 2: document
    ----------

        |b f|

        f := '/home2/stx/lib/doc/online/english/TOP.html'.
        b := Dictionary new.

        b at:#date        put:(Date today printString).
        b at:#url         put:f.
        b at:#exception   put:'no exception information'.
        b at:#message     put:'no message'.

        HTMLDocumentPrinter print:f contents:nil bindings:b style:nil

    example 3: book
    ----------

        HTMLDocumentPrinter printBook:'/home2/stx/lib/doc/online/english/programming/ca.html'

"
!

test
        HTMLDocumentPrinter printBook:'/home2/stx/lib/doc/online/english/programming/ca.html'
! !

!HTMLDocumentPrinter  class methodsFor:'instance creation'!

style:aPrintStyle bindings:someBindings
    "creation of an instance; on error nil is returned
    "
    ^ self new style:aPrintStyle bindings:someBindings
! !

!HTMLDocumentPrinter  class methodsFor:'printing documents'!

print:aFile
    "print a document. On success true is returned otherwise false
    "
    ^ self print:aFile contents:nil bindings:nil style:nil

!

print:aFileOrDirectory contents:aContents bindings:someBindings style:aPrintStyle
    "print a document or contents. On success true is returned
     otherwise false
    "
    |inst|

    inst := self style:aPrintStyle bindings:someBindings.

    inst notNil ifTrue:[
        ^ inst printDocument:aFileOrDirectory contents:aContents
    ].
    ^ false
!

printBook:aBook
    "printing a book (collection of documents). On success
     true is returned otherwise false
    "
    ^ self printBook:aBook bindings:nil style:nil
!

printBook:aBook bindings:someBindings style:aPrintStyle
    "printing a book (collection of documents). On success
     true is returned otherwise false
    "
    |inst|

    inst := self style:aPrintStyle bindings:someBindings.

    inst notNil ifTrue:[
        ^ inst printBook:aBook
    ].
    ^ false


! !

!HTMLDocumentPrinter methodsFor:'accessing'!

bindings
    "accessing bindings 
    "
    ^ bindings
!

bindings:someBindings
    "changing bindings
    "
    bindings := someBindings
!

style
    "accessing printing style
    "
    ^ style
!

style:aStyle
    "change printing style
    "
    (style := aStyle) isNil ifTrue:[
        style := self defaultStyle
    ].
! !

!HTMLDocumentPrinter methodsFor:'instance creation'!

defaultStyle
    "initialize default style
    "
    |style fontSize|

    style := IdentityDictionary "HTMLStyle" new.

    fontSize := 10.
    style at:#fontSize             put:fontSize.

    style at:#normalFont           put:(Font family:'times' face:'medium' style:'roman' size:fontSize).
    style at:#boldNormalFont       put:(Font family:'times' face:'bold'   style:'roman' size:fontSize).
    style at:#italicNormalFont     put:(Font family:'times' face:'medium' style:'italic' size:fontSize).
    style at:#boldItalicNormalFont put:(Font family:'times' face:'bold'   style:'italic' size:fontSize).

    style at:#codeFont             put:(Font family:'courier' face:'medium' style:'roman' size:fontSize).
    style at:#boldCodeFont         put:(Font family:'courier' face:'bold'   style:'roman' size:fontSize).
    style at:#italicCodeFont       put:(Font family:'courier' face:'medium' style:'oblique' size:fontSize).
    style at:#boldItalicCodeFont   put:(Font family:'courier' face:'bold'   style:'oblique' size:fontSize).

    style at:#normalSmallFont      put:(Font family:'times' face:'medium' style:'roman' size:fontSize-2).

    style at:#h1Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.8) rounded).
    style at:#h2Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.4) rounded).
    style at:#h3Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.2) rounded).
    style at:#h4Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize)).
    style at:#h5Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 0.8) rounded).
    style at:#h6Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 0.8) rounded).

    style at:#activeAnchorFGColor  put:(Color red).
    style at:#activeAnchorBGColor  put:(Color white).
    style at:#anchorFGColor        put:(Color blue).
    style at:#exampleFGColor       put:(Color red:40 green:0 blue:0).
    style at:#textFGColor          put:(Color black).
    style at:#visitedAnchorFGColor put:(Color red:15 green:2 blue:36).
    style at:#fontSize             put:10.

    Printer supportsColor ifFalse:[
        style at:#activeAnchorFGColor  put:(Color black).
        style at:#activeAnchorBGColor  put:(Color white).
        style at:#anchorFGColor        put:(Color black).
        style at:#exampleFGColor       put:(Color black).
        style at:#textFGColor          put:(Color black).
        style at:#visitedAnchorFGColor put:(Color black).
    ].
    ^ style

!

style:aStyle bindings:someBindings
    "initialize instance
    "
    bindings := someBindings.
    style    := aStyle.

    style isNil ifTrue:[
        style := self defaultStyle
    ].
    Printer supportsPostscript ifTrue:[
        ^ self
    ].
    self warn:'Sorry, can only print on postscript printers.'.
    ^ nil.
! !

!HTMLDocumentPrinter methodsFor:'printing'!

printBook:aFile
    "printing a book (collection of documents). On success
     true is returned otherwise false
    "
    |file psgc painter book|

    (file := self stringAsFilename:aFile) isNil ifTrue:[^ false].

    file isDirectory ifTrue:[
        self warn:'cannot print a directory: ', file pathName.
      ^ false
    ].

    directory := file directory.
    book      := self parse:file.

    (book isNil or:[(psgc := self openPSGraphicContext) isNil]) ifTrue:[
        ^ false
    ].

    [self printBook:book on:psgc] valueNowOrOnUnwindDo:[
        psgc close.
    ].
    ^ true
!

printDocument:fileOrDirectory contents:aContents
    "printing a document or contents. In case that the contents is not nil,
     the fileOrDirectory instance is used as the top directory for accessing
     images or files. On success true is returned otherwise false
    "
    |file psgc doc|

    file := self stringAsFilename:fileOrDirectory.

    file notNil ifTrue:[
        file isDirectory ifTrue:[directory := file] 
                        ifFalse:[directory := file directory].

        aContents notNil ifTrue:[doc := self parse:aContents]
                        ifFalse:[doc := self parse:file].

        (doc notNil and:[(psgc := self openPSGraphicContext) notNil]) ifTrue:[
            [self printDocument:doc on:psgc] valueNowOrOnUnwindDo:[
                psgc close.
            ].
            ^ true
        ]
    ].
    ^ false
! !

!HTMLDocumentPrinter methodsFor:'private'!

XXelement:element for:aPSGraphicContext
    "prepare a document derived from element for printing. On success
     the document is writen to the graphic context and true is returned.
     Otherwise false returns.
    "
    |file doc idx savDir headers|

    headers := #( h1 h2 h3 h4 h5 h6).

    file := self stringAsFilename:(element hrefString).

    (doc := self parse:file) isNil ifTrue:[
        ^ false
    ].
    idx       := element numericParameterFor:'HEADERINDEX' default:0.
    savDir    := directory.
    directory := file directory.

    idx == 0 ifFalse:[
        doc keysAndValuesDo:[:i :e||s ne|
            e type notNil ifTrue:[
                s := e type asString.
                (s size == 2 and:[(s at:1) == $h and:[(s at:2) isDigit]]) ifTrue:[
                    s := ((s at:2) digitValue) + idx.

                    s < 1 ifTrue:[s := 1] 
                         ifFalse:[s > 6 ifTrue:[s := 6]].

                    ne := e deepCopy.
                    ne type:(headers at:s).
                    doc at:i put:ne.
                ]
            ]
        ]
    ].

    self printDocument:doc on:aPSGraphicContext.
    directory := savDir.
    ^ true.
!

element:element for:aPSGraphicContext
    "prepare a document derived from element for printing. On success
     the document is writen to the graphic context and true is returned.
     Otherwise false returns.
    "
    |file doc hdrOffset savDir headers|

    headers := #(h1 h2 h3 h4 h5 h6 /h1 /h2 /h3 /h4 /h5 /h6 ).

    file := self stringAsFilename:(element hrefString).

    (doc := self parse:file) isNil ifTrue:[
        ^ false
    ].
    hdrOffset := element numericParameterFor:'HEADERINDEX' default:0.
    savDir    := directory.
    directory := file directory.

    hdrOffset == 0 ifFalse:[
        doc := doc collect:[:e||i min max|
            i := headers indexOf:(e type).
            i ~~ 0 ifFalse:[
                e
            ] ifTrue:[
                i <= 6 ifTrue:[min := 1. max :=  6]
                      ifFalse:[min := 7. max := 12].

                i := i + hdrOffset.
                i < min ifTrue:[i := min]
                       ifFalse:[i > max ifTrue:[i := max]].

                HTMLMarkup for:((headers at:i) asString)
            ]
        ]
    ].
    self printDocument:doc on:aPSGraphicContext.
    directory := savDir.
    ^ true.
!

openPSGraphicContext
    "open a PSGraphicsContext on a stream. On success the
     context is returned otherwise nil
    "
    |stream psgc|

    stream := Printer newNative.

    stream notNil ifTrue:[
        ^ PSGraphicsContext on:stream extent:(1.0 @ 1.0)
    ].
    self warn:'cannot open printer stream'.
    ^ nil
        
!

parse:stringOrFile
    "parse string.or file. On success the document is returned
     otherwise nil.
    "
    |stream doc|

    stringOrFile notNil ifTrue:[
        stringOrFile isString ifTrue:[
            doc := HTMLParser new parseText:stringOrFile withBindings:bindings
        ] ifFalse:[
            stream := stringOrFile readStream.
            stream notNil ifTrue:[
                doc := HTMLParser new parseText:stream withBindings:bindings.
                stream close.
            ] ifFalse:[
                self warn:'cannot open stream for: ', stringOrFile pathName
            ]
        ]
    ].
    (doc isNil or:[doc isEmpty]) ifFalse:[^ doc]
                                  ifTrue:[^ nil].

        
!

stringAsFilename:aString
    "returns string as an absolute filename or nil
    "
    |file strLower strUpper|

    aString notNil ifTrue:[
        file := aString asFilename.
        file exists ifTrue:[^ file asAbsoluteFilename].

        strLower := aString asString asLowercase.
        file     := strLower asFilename.
        file exists ifTrue:[^ file asAbsoluteFilename].

        strUpper := aString asString asUppercase.
        file     := strUpper asFilename.
        file exists ifTrue:[^ file asAbsoluteFilename].

        directory notNil ifTrue:[
            file := directory construct:aString.
            file exists ifTrue:[^ file asAbsoluteFilename].

            file := directory construct:strLower.
            file exists ifTrue:[^ file asAbsoluteFilename].

            file := directory construct:strUpper.
            file exists ifTrue:[^ file asAbsoluteFilename].
        ].
        self warn:'not a valid filename: ', aString asString.
    ].
    ^ nil
! !

!HTMLDocumentPrinter methodsFor:'private printer interface'!

imageFor:urlString now:now
    "returns image assigned to urlString or nil
    "
    |file|

    (file := self stringAsFilename:urlString) notNil ifTrue:[
        ^ Image fromFile:(file pathName)
    ].
    ^ nil
!

visitTest:urlString action:actionString
    ^ false

! !

!HTMLDocumentPrinter methodsFor:'private printing'!

printBook:book on:aPSGraphicContext
    "print documents of book to a grafic context
    "
    |newPage success|

    success := true.
    newPage := false.

    book do:[:e|
        e type == #a ifTrue:[
            newPage ifTrue:[
                aPSGraphicContext startPage.
            ].
            (newPage := self element:e for:aPSGraphicContext) ifFalse:[
                success := false
            ].
        ]
    ].
    ^ success
!

printDocument:document on:aPSGraphicContext
    "print document to a grafic context
    "
    |painter height|

    height  := 5000000.

    painter := HTMLDocumentPainter new.
    painter format:document for:aPSGraphicContext resolver:self style:style.
    painter displayX:0 y:0 width:9000 height:height on:aPSGraphicContext.

    "/ height := painter height.   (page height).

! !

!HTMLDocumentPrinter  class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/HTMLDocumentPrinter.st,v 1.5 1996-09-10 11:51:34 ca Exp $'
! !
