View subclass:#HTMLDocumentView
	instanceVariableNames:'document homeDocument homeText showImages topDirectoryName
		currentDirectoryName enableExecution hRefHistory
		currentPercentage visitedActions visitedUrls currentDocument
		currentAction painter currentLocalAnchor localImages uriHolder
		infoHolder linkButtonPanel currentURL style printStyle
		formatContext printing'
	classVariableNames:'CachedImages OpenHTMLWidgets OpenHelpViews HTMLExtension
		ImageFlushProcess ImageFlushDelay InternalImageTable
		FileExtensionTable FileExecutorTable'
	poolDictionaries:''
	category:'System-Documentation'
!

!HTMLDocumentView  class methodsFor:'documentation'!

examples
"
    setup an HTML view in some other view:
                                                                [exBegin]
    |top v document|

    top := StandardSystemView extent:200@500.
    v := HVScrollableView for:HTMLDocumentView miniScrollerH:true in:top.
    v origin:0.0@ 0.0 corner:1.0@1.0.
    top openAndWait.

    v homeDocument:'../../doc/online/english/TOP.html'.
                                                                [exEnd]


    the same, with different contents:
                                                                [exBegin]
    |top v document|

    top := StandardSystemView extent:200@500.
    v := HVScrollableView for:HTMLDocumentView miniScrollerH:true in:top.
    v origin:0.0@ 0.0 corner:1.0@1.0.
    top openAndWait.

    v homeDocument:'test.html'.
                                                                [exEnd]


  how about something like this for your applications help:
                                                                [exBegin]
     |top panel list bottom htmlView|

     top := StandardSystemView new.
     top extent:(Display extent // 2).

     panel := VariableVerticalPanel origin:0.0@0.0 corner:1.0@1.0 in:top.

     list := ScrollableView for:FileSelectionList origin:0.0@0.0 corner:1.0@0.25 in:panel.
     list directory:'../../doc/online/english'.
     list action:[:arg | 
                        htmlView setTopDirectoryName:(list directory pathName).
                        htmlView showFileDocument:(list selectedPathname)
                 ].
     list matchBlock:[:name | '*.html' match:name].

     bottom := View origin:0.0@0.25 corner:1.0@1.0 in:panel.
     htmlView := HTMLDocumentView 
                        onFile:'../../doc/online/english/TOP.html' 
                        in:bottom.

     top open
                                                                [exEnd]

  it is also possible, to stuff HTML source into the view:
                                                                [exBegin]
     HTMLDocumentView
         openFullOnText:'
<html>
<body>
<h1>chapter 1</h1>
<h2>sub chapter 1.1</h2>
<h1>chapter 2</h1>
<h2>sub chapter 2.1</h2>
<h2>sub chapter 2.2</h2>
<h2>sub chapter 2.3</h2>
</body>
</html>
' 
                                                                [exEnd]

  and, the same in a subview:
                                                                [exBegin]
     |top panel list bottom htmlView|

     top := StandardSystemView new.
     top extent:(Display extent // 2).

     panel := VariableVerticalPanel origin:0.0@0.0 corner:1.0@1.0 in:top.

     list := ScrollableView for:FileSelectionList origin:0.0@0.0 corner:1.0@0.25 in:panel.
     list directory:'../../doc/online/english'.
     list action:[:arg | 
                        htmlView setTopDirectoryName:(list directory pathName).
                        htmlView showFileDocument:(list selectedPathname)
                 ].
     list matchBlock:[:name | '*.html' match:name].

     bottom := View origin:0.0@0.25 corner:1.0@1.0 in:panel.
     htmlView := HTMLDocumentView 
                        onText:'
<html>
<body>
<h1>chapter 1</h1>
<h2>sub chapter 1.1</h2>
<h1>chapter 2</h1>
<h2>sub chapter 2.1</h2>
<h2>sub chapter 2.2</h2>
<h2>sub chapter 2.3</h2>
</body>
</html>
'
                        in:bottom.

     top open
                                                                [exEnd]


"

! !

!HTMLDocumentView  class methodsFor:'initialization'!

initialize
    OperatingSystem isMSDOSlike ifTrue:[
        HTMLExtension := '.htm'
    ] ifFalse:[
        HTMLExtension := '.html'
    ].
    ObjectMemory addDependent:self.
    ImageFlushDelay := 300.
    self initializeInternalImageTable.
    self initializeFileExtensionTable.

    "
     self initialize
    "

    "Created: 20.5.1996 / 20:29:56 / cg"
    "Modified: 10.7.1996 / 14:38:55 / cg"
!

initializeFileExecutorTable
    FileExecutorTable isNil ifTrue:[
        FileExecutorTable := Dictionary new.
        
        FileExecutorTable at:'text-plain'  put:self.
        FileExecutorTable at:'text-html'   put:self.
    ].

    "
     FileExecutorTable := nil.
     self initializeFileExecutorTable
    "

    "Modified: 8.7.1996 / 19:15:28 / cg"
    "Created: 10.7.1996 / 14:41:02 / cg"
!

initializeFileExtensionTable
    FileExtensionTable isNil ifTrue:[
        FileExtensionTable := Dictionary new.
        
        FileExtensionTable at:'txt'  put:#('internal-gopher-text'   'text-plain').
        FileExtensionTable at:'htm'  put:#('internal-gopher-text'   'text-html').
        FileExtensionTable at:'html' put:#('internal-gopher-text'   'text-html').

        FileExtensionTable at:'gif'  put:#('internal-gopher-image'  'image-gif').
        FileExtensionTable at:'tiff' put:#('internal-gopher-image'  'image-tiff').
        FileExtensionTable at:'xbm'  put:#('internal-gopher-image'  'image-xbm').
        FileExtensionTable at:'xpm'  put:#('internal-gopher-image'  'image-xbm').

        FileExtensionTable at:'au'   put:#('internal-gopher-sound'  'misc-sound-au').
        FileExtensionTable at:'dir'  put:#('internal-gopher-menu'   'misc-dir').
        FileExtensionTable at:'exe'  put:#('internal-gopher-binary' 'misc-binary').
    ].

    "
     FileExtensionTable := nil.
     self initializeFileExtensionTable
    "

    "Created: 8.7.1996 / 18:50:51 / cg"
    "Modified: 10.7.1996 / 14:43:00 / cg"
!

initializeInternalImageTable
    InternalImageTable isNil ifTrue:[
        InternalImageTable := Dictionary new.
        
        InternalImageTable at:'internal-gopher-unknown' put:'file_unknown.xpm'.
        InternalImageTable at:'internal-gopher-text'    put:'file_text.xpm'.
        InternalImageTable at:'internal-gopher-image'   put:'file_image.xpm'.
        InternalImageTable at:'internal-gopher-sound'   put:'file_sound.xpm'.
        InternalImageTable at:'internal-gopher-menu'    put:'file_dir.xpm'.
        InternalImageTable at:'internal-gopher-binary'  put:'file_binary.xpm'.
    ].

    "
     InternalImageTable := nil.
     self initializeInternalImageTable
    "

    "Created: 20.5.1996 / 20:29:56 / cg"
    "Modified: 8.7.1996 / 18:49:12 / cg"
!

update:something with:aParameter from:changedObject
    (something == #save) ifTrue:[
        "
         smalltalk is about to save an image - dont save cachedImages
        "
        CachedImages := nil.        
    ].

    "Created: 15.6.1996 / 16:42:07 / cg"
! !

!HTMLDocumentView  class methodsFor:'cleanup'!

flushCachedImages
    CachedImages := nil

    "Created: 3.7.1996 / 13:53:13 / cg"
!

lowSpaceCleanup
    self flushCachedImages

    "Created: 20.5.1996 / 13:31:58 / cg"
    "Modified: 3.7.1996 / 13:53:18 / cg"
! !

!HTMLDocumentView  class methodsFor:'defaults'!

defaultHTMLExtension
    ^ HTMLExtension

    "
     self defaultHTMLExtension  
    "

    "Modified: 20.5.1996 / 19:14:39 / cg"
    "Created: 20.5.1996 / 20:29:29 / cg"
!

documentFileFor:relativePath
    "given a relataive path in the documentation hierarchy,
     find the files absolute path - first trying the current language,
     if not found, try an english version"

    |file lang|

    lang := Smalltalk language.

    file := Smalltalk getSystemFileName:'doc/online/' , lang , '/' , relativePath.
    file isNil ifTrue:[
        file := Smalltalk getSystemFileName:'../../doc/online/' , lang , '/' , relativePath.
        file isNil ifTrue:[
            file := Smalltalk getSystemFileName:'doc/online/english/' , relativePath.
            file isNil ifTrue:[
                file := Smalltalk getSystemFileName:'../../doc/online/english/' , relativePath.
            ]    
        ]
    ].

    ^ file

    "
     Smalltalk language:#english.
     self documentFileFor:'TOP.html' 
     self documentFileFor:'programming/viewintro.html' 

     Smalltalk language:#german.
     self documentFileFor:'TOP.html'  
     self documentFileFor:'programming/viewintro.html' 
    "

    "Created: 20.5.1996 / 20:29:29 / cg"
    "Modified: 5.8.1996 / 18:56:39 / cg"
!

helpFileFor:relativePath
    "given a relataive path in the helpFile hierarchy,
     find the files absolute path - first trying the current language,
     if not found, try an english version"

    |file lang|

    lang := Smalltalk language.

    file := Smalltalk getSystemFileName:'help/' , lang , '/' , relativePath.
    file isNil ifTrue:[
        file := Smalltalk getSystemFileName:'doc/online/' , lang , '/help' , relativePath.
        file isNil ifTrue:[
            file := Smalltalk getSystemFileName:'help/english/' , relativePath.
            file isNil ifTrue:[
                file := Smalltalk getSystemFileName:'doc/online/english/help/' , relativePath.
                file isNil ifTrue:[
                    file := Smalltalk getSystemFileName:'../../doc/online/english/help/' , relativePath.
                ]
            ]
        ]
    ].

    ^ file

    "Created: 20.5.1996 / 20:29:29 / cg"
! !

!HTMLDocumentView  class methodsFor:'startup'!

onFile:aDocumentName in:aView
    "create a documentationView on aDocumentName in some other view.
     A ScrollWrapper is always created.
     Since the html view runs on another logical workstation (the XtInterface), 
     this html-view runs under control of a separate process 
     (which may be confusing).
     For technical reasons (Xtlib), the view must be already created
     when the html view is created - therefore, the create is done here;
     this means, that the given topView must have a valid superView
     when this is called - otherwise, its created as a topView."

    ^ self
        onFile:aDocumentName text:nil top:nil in:aView

    "
     |top panel list bottom htmlView|

     top := StandardSystemView new.
     panel := VariableVerticalPanel origin:0.0@0.0 corner:1.0@1.0 in:top.
     list := ScrollableView for:FileSelectionList origin:0.0@0.0 corner:1.0@0.25 in:panel.
     list directory:'../../doc/online/english'.
     list action:[:arg | 
                        htmlView setTopDirectoryName:(list directory pathName).
                        htmlView showFileDocument:(list selectedPathname)
                 ].
     list matchBlock:[:name | '*.html' match:name].
     bottom := View origin:0.0@0.25 corner:1.0@1.0 in:panel.
     htmlView := HTMLDocumentView onFile:'../../doc/online/english/TOP.html' in:bottom.

     top open
    "

    "Created: 20.5.1996 / 20:29:56 / cg"
    "Modified: 10.9.1996 / 15:17:55 / cg"
!

onFile:aFileName text:htmlText top:topDirectory in:top
    "create a documentationView on aDocumentName or htmlText in
     some other view. A ScrollWrapper is always created.
     Helper for onFile:in: and onText:in:"

    |wrapper frame scr html f|

    scr := HVScrollableView for:self miniScrollerH:true miniScrollerV:false in:top.
    scr origin:0.0@0.0 corner:1.0@1.0.
    html := scr scrolledView.

    aFileName isNil ifTrue:[
        html setTopDirectoryName:topDirectory.
        html homeText:htmlText.
    ] ifFalse:[        
        f := aFileName asFilename.
        topDirectory notNil ifTrue:[
            html setTopDirectoryName:topDirectory.
        ] ifFalse:[
            html setTopDirectoryName:(f directoryName).
        ].
        html homeDocument:(f baseName).
    ].
    ^ html

    "Modified: 26.4.1996 / 16:35:52 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

onText:htmlText in:aView
    "create a documentationView on htmlText (must be html) in some other view.
     A ScrollWrapper is always created.
     Since the html view runs on another logical workstation (the XtInterface), 
     this html-view runs under control of a separate process 
     (which may be confusing).
     For technical reasons (Xtlib), the aViewView must be already created
     when the html view is created - therefore, the create is done here;
     this means, that the given view must have a valid superView
     when this is called - otherwise, its created as a TopView."

    ^ self
        onFile:nil text:htmlText top:nil in:aView

    "
     |top panel list bottom htmlView|

     top := StandardSystemView new.
     panel := VariableVerticalPanel origin:0.0@0.0 corner:1.0@1.0 in:top.
     list := ScrollableView for:FileSelectionList origin:0.0@0.0 corner:1.0@0.25 in:panel.
     list directory:'../../doc/online/english'.
     list action:[:arg | 
                        htmlView setTopDirectoryName:(list directory pathName).
                        htmlView showFileDocument:(list selectedPathname)
                 ].
     list matchBlock:[:name | '*.html' match:name].
     bottom := View origin:0.0@0.25 corner:1.0@1.0 in:panel.
     htmlView := self 
                    onText:'
<html>
<body>
<h1>some text</h1>
</body>
</html>
' 
                    in:bottom.

     top open
    "

    "Modified: 26.4.1996 / 17:02:37 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

open
    ^ self openOnFile:'../../doc/online/english/TOP.html'

    "
     self open
    "

    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFull
    "open a full documentation view on the top documentation page.
     The actual page displayed depends on the language setting;
     i.e. it is 'doc/online/<language>/TOP.html'."

    self openFullOnDocumentationFile:'TOP.html'

    "
     self openFull
    "
    "
     Language := #german.
     self openFull
    "
    "
     Language := #english.
     self openFull
    "

    "Modified: 26.4.1996 / 12:21:39 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFullOnDocumentationFile:relativePath
    "open a full documentation view on a file, given its relative
     path in the documentation hierachy.
     The actual page displayed depends on the language setting;
     i.e. it is 'doc/online/<language>/relativePath'."

    |file|

    file := self documentFileFor:relativePath.
    file notNil ifTrue:[
        ^ self openFullOnFile:file
    ].
    self warn:'no file: doc/online/english/' , relativePath.

    "
     self openFullOnDocumentationFile:'TOP.html'
     self openFullOnDocumentationFile:'programming/TOP.html'
     self openFullOnDocumentationFile:'foobar'
    "
    "
     Language := #german.
     self openFullOnDocumentationFile:'TOP.html'
    "
    "
     Language := #english.
     self openFullOnDocumentationFile:'TOP.html'
    "

    "Modified: 26.4.1996 / 17:25:57 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFullOnFile:aDocumentName
    "open a full documentation view on a file, given its pathName"

    |display|

    display := Screen current.
    ^ self 
        openFullOnFile:aDocumentName 
        extent:(display width // 2) @ (display height-50) 
        title:'ST/X Documentation view'

    "
     self openFullOnFile:'../../doc/online/english/TOP.html'
    "

    "Modified: 26.4.1996 / 12:28:27 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFullOnFile:aDocumentName extent:extent title:title
    "open a full documentation view with extent and title,
     on a file, given its pathName.
     Add a standard button panel."

    ^ self
        openFullOnFile:aDocumentName
        text:nil
        top:nil
        extent:extent 
        title:title
        buttonSpec:#( 
                        'quit'           #menu_quit 
                        'reload'         #menu_reload 
                        'home'           #menu_home 
                        'back'           #menu_back 
                        'print'          #menu_print
"/                        'hardcopy'       #menu_printPage
                        nil              nil
                        'source'         #menu_source
                        'settings ...'   #menu_settings 
                        nil              nil
                        'help'           #menu_help
                    )
        uriDisplay:true
        infoDisplay:false "true"

    "Created: 20.5.1996 / 20:29:56 / cg"
    "Modified: 5.8.1996 / 18:30:59 / stefan"
    "Modified: 10.9.1996 / 15:00:40 / cg"
!

openFullOnFile:aDocumentName text:htmlText top:topDirectory extent:extent title:title buttonSpec:buttonSpec info:info
    "open a full documentation view with extent and title,
     on a file, given its pathName.
     The buttonSpec argument specifies which buttons should be added to
     the panel - for a help viewer, only a subset of the complete set
     is usually passed."

    ^ self
        openFullOnFile:aDocumentName 
        text:htmlText 
        top:topDirectory 
        extent:extent 
        title:title 
        buttonSpec:buttonSpec 
        uriDisplay:info
        infoDisplay:info

!

openFullOnFile:aDocumentName text:htmlText top:topDirectory extent:extent title:title buttonSpec:buttonSpec uriDisplay:showUri infoDisplay:showInfo
    "open a full documentation view with extent and title,
     on a file, given its pathName.
     The buttonSpec argument specifies which buttons should be added to
     the panel - for a help viewer, only a subset of the complete set
     is usually passed."

    |top buttonPanel buttonPanel2 wrapper frame scr html foc 
     infoView infoLabel uriView uriLabel infoHolder uriHolder spacing 
     bPanelPref y1|

    spacing := View viewSpacing.

    top := StandardSystemView extent:extent.
    top label:title.

    buttonSpec notNil ifTrue:[
        buttonPanel := HorizontalPanelView origin:0@0 corner:1.0@0.0 in:top.
        foc := OrderedCollection new.

        buttonSpec pairWiseDo:[:label :selector |
            |b sep|

            label isNil ifTrue:[
                sep := SimpleView extent:50@10.
                sep borderWidth:0; level:0.
                buttonPanel add:sep
            ] ifFalse:[
                b := Button label:(self classResources string:label).
                b action:[html perform:selector].
                buttonPanel add:b.
                foc add:b.
            ]
        ].
        buttonPanel bottomInset:(buttonPanel preferredExtent y negated).
        buttonPanel horizontalLayout:#leftSpace.
        bPanelPref := buttonPanel preferredExtent.
        y1 := bPanelPref y.
    ] ifFalse:[
        y1 := 0.
    ].

    buttonPanel2 := HorizontalPanelView origin:0@0 corner:1.0@0.0 in:top.
    buttonPanel2 topInset:y1.
    buttonPanel2 beInvisible.

    frame := View origin:0.0@0.0 corner:1.0@1.0 in:top.

    showUri ifTrue:[
        uriLabel := Label in:top.
        uriLabel label:(self classResources string:'location:').
        uriLabel adjust:#left; borderWidth:0.
        uriLabel layout:(LayoutFrame new
                                leftFraction:0.0 offset:spacing;
                                rightFraction:0.0 offset:(uriLabel preferredExtent x);
                                topFraction:0.0 offset:y1;
                                bottomFraction:0.0 offset:(y1 + uriLabel preferredExtent y + (spacing * 2))).

        uriHolder := '' asValue.

        uriView := Label in:top.
        uriView labelChannel:uriHolder.
        uriView adjust:#left; borderWidth:0.
        uriView layout:(LayoutFrame new
                                leftFraction:0.0 offset:(uriLabel preferredExtent x + spacing);
                                rightFraction:1.0 offset:spacing negated;
                                topFraction:0.0 offset:y1;
                                bottomFraction:0.0 offset:(y1 + uriLabel preferredExtent y + (spacing * 2))).
        uriView sizeFixed:true.

        StyleSheet is3D ifTrue:[
            uriView level:-1.
        ].

        frame topInset:(y1 + uriView preferredExtent y + (spacing * 2)).
    ] ifFalse:[
        frame topInset:(y1).
    ].

    showInfo ifTrue:[
        infoLabel := Label in:top.
        infoLabel label:'ref:'.
        infoLabel adjust:#left.
        infoLabel layout:(LayoutFrame new
                                leftFraction:0.0 offset:spacing;
                                rightFraction:0.0 offset:(infoLabel preferredExtent x);
                                topFraction:1.0 offset:(infoLabel preferredExtent y negated - spacing);
                                bottomFraction:1.0 offset:0).

        infoHolder := '' asValue.

        infoView := Label in:top.
        infoView labelChannel:infoHolder.
        infoView adjust:#left.
        infoView layout:(LayoutFrame new
                                leftFraction:0.0 offset:(infoLabel preferredExtent x + spacing);
                                rightFraction:1.0 offset:spacing negated;
                                topFraction:1.0 offset:(infoLabel preferredExtent y negated - spacing);
                                bottomFraction:1.0 offset:0).
        infoView sizeFixed:true.

        StyleSheet is3D ifTrue:[
            infoView level:-1.
        ].

        frame bottomInset:(infoLabel preferredExtent y + (spacing * 2)).
    ] ifFalse:[
        frame bottomInset:0.
    ].

    html := self 
                onFile:nil 
                text:htmlText 
                top:topDirectory
                in:frame.


    showInfo ifTrue:[
        html infoHolder:infoHolder.
    ].
    showUri ifTrue:[
        html uriHolder:uriHolder.
    ].
    html linkButtonPanel:buttonPanel2.
    aDocumentName notNil ifTrue:[
       html homeDocument:aDocumentName.
    ].
    top open.
    ^ html.

    "
     self openFullOnFile:'../../doc/online/english/TOP.html'
    "

    "Created: 13.6.1996 / 11:07:28 / cg"
    "Modified: 10.9.1996 / 14:15:05 / cg"
!

openFullOnHelpFile:relativePath
    "open a full documentationView as helpView (not all buttons are
     present) on relativePath, which is the path relative to the 
     doc/online/help directory. If aRelativePath ends in '.html', 
     that filename is taken; otherwise, it is interpreted as a directory
     name and a file named 'TOP.html' is tried there."

    |v file relativePathTop display|

    OpenHelpViews isNil ifTrue:[
        OpenHelpViews := Dictionary new.
    ].

    v := OpenHelpViews at:relativePath ifAbsent:nil.
    v notNil ifTrue:[
        v topView raiseDeiconified.
        ^ self
    ].

    (relativePath endsWith:'.html') ifFalse:[
        relativePathTop := relativePath , '/TOP.html'.
    ] ifTrue:[
        relativePathTop := relativePath
    ].

    file := self helpFileFor:relativePathTop.
    file notNil ifTrue:[
        display := Screen current.
        v := self 
                openFullOnFile:file
                text:nil
                top:nil
                extent:(display extent // 2)
                title:'ST/X Help'
                buttonSpec:#( 
                            'close'      #menu_quit 
                            'back'       #menu_back
                            nil          nil    
                            'print'      #menu_print
                        )
                uriDisplay:false
                infoDisplay:false.
        OpenHelpViews at:relativePath put:v.
        ^ v
    ].
    self warn:'missing help file: help/english/' , relativePathTop.

    "
     self openFullOnHelpFile:'Launcher/compilerSettings.html'
    "
    "
     Language := #german.
     self openFullOnHelpFile:'Launcher/compilerSettings.html'
    "
    "
     Language := #english.
     self openFullOnHelpFile:'Launcher/compilerSettings.html'
    "

    "Created: 20.5.1996 / 20:29:56 / cg"
    "Modified: 13.6.1996 / 11:08:05 / cg"
    "Modified: 9.9.1996 / 22:40:41 / stefan"
!

openFullOnText:htmlText
    "open a full documentation view on some html-text."

    ^ self
        openFullOnText:htmlText inDirectory:nil
    "
     self
         openFullOnText:'
<html>
<body>
<h1>chapter 1</h1>
<h2>sub chapter 1.1</h2>
<h1>chapter 2</h1>
<h2>sub chapter 2.1</h2>
<h2>sub chapter 2.2</h2>
<h2>sub chapter 2.3</h2>
</body>
</html>
' 
    "

    "Modified: 26.4.1996 / 16:59:18 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFullOnText:htmlText inDirectory:topDirectory
    "open a full documentation view on some html-text.
     For file-HREFS, use aTopDirectory as starting point."

    |display|

    display := Screen current.
    ^ self 
        openFullOnText:htmlText 
        top:topDirectory
        extent:(display width // 2) @ (display height-50) 
        title:'ST/X Documentation view'

    "
     self
         openFullOnText:'
<html>
<body>
<h1>chapter 1</h1>
<h2>sub chapter 1.1</h2>
<h1>chapter 2</h1>
<h2>sub chapter 2.1</h2>
<h2>sub chapter 2.2</h2>
<h2>sub chapter 2.3</h2>
</body>
</html>
' 
    "

    "Modified: 26.4.1996 / 17:04:23 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openFullOnText:htmlText top:topDirectory extent:extent title:title
    "open a full documentation view with extent and title,
     on a file, given its pathName.
     Add a standard button panel."

    ^ self
        openFullOnFile:nil
        text:htmlText
        top:topDirectory
        extent:extent 
        title:title
        buttonSpec:#( 
                        'quit'           #menu_quit 
                        'reload'         #menu_reload 
                        'home'           #menu_home 
                        'back'           #menu_back 
                        'print'          #menu_print
"/                        'hardcopy'     #menu_printPage
                        nil              nil
                        'source'         #menu_source
                        'settings ...'   #menu_settings 
                        nil              nil
                        'help'           #menu_help
                    )
        uriDisplay:true
        infoDisplay:true.

    "Created: 20.5.1996 / 20:29:57 / cg"
    "Modified: 5.8.1996 / 18:30:51 / stefan"
    "Modified: 10.9.1996 / 15:01:04 / cg"
!

openOn:aDocumentName
    "open a full sized documentationView on aDocumentName"

    ^ self openFullOnFile:aDocumentName

    "
     self openOn:'../../doc/online/english/TOP.html'
    "

    "Modified: 26.4.1996 / 16:45:52 / cg"
    "Created: 20.5.1996 / 20:29:56 / cg"
!

openOnFile:aDocumentName
    "open a small documentationView on aDocumentName"

    |top wrapper frame scr html|

    top := StandardSystemView new.
    top extent:400@600.

    html := self onFile:aDocumentName text:nil top:nil in:top.

    top open.

    ^ html.

    "
     self openOnFile:'../../doc/online/english/TOP.html'
    "

    "Modified: 26.4.1996 / 17:01:32 / cg"
    "Created: 20.5.1996 / 20:29:57 / cg"
! !

!HTMLDocumentView methodsFor:'accessing'!

defaultBackgroundColor
    |bg|

    bg := styleSheet colorAt:'htmlBackgroundColor' default:(Color grey:80).
    Screen current hasGreyscales ifFalse:[
        bg := Color white.
    ].
    ^ bg.

    "Modified: 12.6.1996 / 18:21:09 / cg"
!

defaultViewBackgroundColor
    |bg|

    bg := styleSheet colorAt:'htmlBackgroundColor' default:(Color grey:80).
    Screen current hasGreyscales ifFalse:[
        bg := Color white.
    ].
    ^ bg.

    "Modified: 12.6.1996 / 18:21:09 / cg"
    "Created: 12.6.1996 / 18:21:58 / cg"
!

enableExecution:aBoolean
    enableExecution := aBoolean.
    aBoolean ifTrue:[
"/        exampleFgColor := Color red:40 green:0 blue:0.
    ] ifFalse:[
"/        exampleFgColor := Color black.
    ].
"/    realized ifTrue:[
"/        exampleFgColor := exampleFgColor on:device.
"/        self setArg:'exampleAnchorColor' value:exampleFgColor colorId.
"/    ].

    "Created: 19.7.1996 / 14:30:44 / cg"
!

getDocumentFilename:urlString
    |fileName|

    (urlString startsWith:'/') ifTrue:[
        fileName := urlString
    ] ifFalse:[
        "/ netscape compatibility
        (urlString startsWith:'internal-') ifTrue:[
            fileName := InternalImageTable at:urlString ifAbsent:nil.
            fileName notNil ifTrue:[
                fileName := Smalltalk getBitmapFileName:fileName.
                fileName notNil ifTrue:[
                    ^ fileName asFilename
                ]
            ]
        ].
        currentDirectoryName isNil ifTrue:[
            ^ urlString asFilename
        ].
        fileName := (currentDirectoryName , '/' , urlString).

        fileName asFilename exists ifFalse:[
            fileName := currentDirectoryName , '/' , urlString asLowercase.
            fileName asFilename exists ifTrue:[
                ^ fileName asFilename
            ].
            fileName := currentDirectoryName , '/' , urlString asUppercase.
            fileName asFilename exists ifTrue:[
                ^ fileName asFilename
            ].
        ].
    ].
    ^ fileName asFilename

    "Created: 17.5.1996 / 17:02:44 / cg"
    "Modified: 5.9.1996 / 12:08:14 / cg"
!

getDocumentSource:anchorHref action:action
    |rest localAnchor method file idx text found|

    anchorHref notNil ifTrue:[
        method := 'file'.
        idx := anchorHref indexOf:$:.
        idx ~~ 0 ifTrue:[
            method := anchorHref copyTo:(idx - 1).
            rest := anchorHref copyFrom:(idx + 1).
        ] ifFalse:[
            rest := anchorHref
        ].

        "/
        "/ rest is everything after initial <method>:
        "/
        idx := rest indexOf:$# startingAt:idx + 1.
        found := false.
        [found or:[idx == 0]] whileFalse:[
            (rest at:idx+1) ~~ $# ifTrue:[
                found := true.
            ] ifFalse:[
                idx := rest indexOf:$# startingAt:idx+2
            ]
        ].
        idx ~~ 0 ifTrue:[
            file := rest copyTo:(idx - 1).
            file isEmpty ifTrue:[
                file := nil
            ].
        ] ifFalse:[
            file := rest
        ].

        action notNil ifTrue:[
            idx := action indexOf:$:.
            idx ~~ 0 ifTrue:[
                method := action copyTo:(idx - 1).
                rest := action copyFrom:(idx + 1).
            ] ifFalse:[
                method := 'doit'.
                rest := action
            ].

            self topView withExecuteCursorDo:[
                method = 'doit' ifTrue:[
                    Object errorSignal catch:[
                        Compiler evaluate:rest
                    ]    
                ] ifFalse:[
                    (method = 'plain') ifTrue:[
                        Object errorSignal catch:[
                            text := Compiler evaluate:rest.
                        ].
                        ^ text
                    ] ifFalse:[
                        (method = 'html') ifTrue:[
                            Object errorSignal catch:[
                                text := Compiler evaluate:rest.
                            ].
                            ^ text
                        ] ifFalse:[
"/                          self halt:'unexpected anchor'
                        ]
                    ]
                ]
            ]
        ] ifFalse:[
            file notNil ifTrue:[
                method = 'file' ifTrue:[
                    ^ self getFileSource:file
                ] ifFalse:[
"/                  self halt:'unexpected anchor'
                ]
            ]
        ].
    ].
    ^ nil

    "Created: 21.5.1996 / 10:07:40 / cg"
!

getFileSource:urlString
    |fileName s text newDoc
     myTopView|

    urlString notNil ifTrue:[
        (myTopView := self topView) withCursor:(Cursor read) do:[

            fileName := self getDocumentFilename:urlString.
            fileName isDirectory ifFalse:[
                s := fileName readStream.
            ].
            s notNil ifTrue:[
                text := s contents asString.
                s close.
                ^ text.
            ].
        ].
    ].

    ^ nil

    "Created: 21.5.1996 / 10:06:21 / cg"
!

homeDocument:aFilename
    homeDocument := aFilename.
    homeText := nil.
    self showFileDocument:aFilename anchor:nil remember:true.

    "Created: 17.5.1996 / 17:05:41 / cg"
    "Modified: 17.5.1996 / 17:05:55 / cg"
!

homeText:someText
    homeDocument := nil.
    homeText := someText.

    self rememberDocument:nil text:someText action:nil inHistory:true.

    currentURL := 'internal'.

    self 
        setText:someText 
        anchor:nil 
        documentName:nil.

    "Modified: 10.7.1996 / 15:44:01 / cg"
!

infoHolder:aValueHolder
    infoHolder := aValueHolder

    "Created: 25.7.1996 / 12:02:58 / cg"
!

linkButtonPanel:aView
    linkButtonPanel := aView

    "Created: 18.6.1996 / 23:05:43 / cg"
!

rememberDocument:document text:text action:action inHistory:yesOrNo
"/ 'remember: ' print. document printCR.

    document notNil ifTrue:[
        yesOrNo ifTrue:[
            self getVerticalPosition.

            hRefHistory isNil ifTrue:[
                hRefHistory := OrderedCollection new
            ].
            hRefHistory addLast:(Array with:document 
                                       with:action 
                                       with:nil
                                       with:currentPercentage
                                       with:text ).
        ].

        action notNil ifTrue:[
            visitedActions isNil ifTrue:[
                visitedActions := Set new.
            ].
            visitedActions add:action.
        ] ifFalse:[
            document notNil ifTrue:[
                visitedUrls isNil ifTrue:[
                    visitedUrls := Set new.
                ].
                visitedUrls add:document.
            ]
        ]
    ].

    "Created: 17.5.1996 / 17:06:39 / cg"
    "Modified: 8.7.1996 / 20:57:21 / cg"
!

setText:text
    "set the text - see example"

    AbortSignal catch:[
        self 
            setText:text 
            header:nil 
            footer:nil 
            id:nil 
            anchor:nil
    ]

    "Modified: 22.4.1996 / 22:41:11 / cg"
    "Created: 17.5.1996 / 17:09:14 / cg"
!

setText:text anchor:anchor
    "set the text - see example"

    AbortSignal catch:[
        self 
            setText:text 
            header:nil 
            footer:nil 
            id:nil 
            anchor:anchor
    ]

    "Modified: 22.4.1996 / 22:41:01 / cg"
    "Created: 17.5.1996 / 17:12:07 / cg"
!

setText:text anchor:anchor documentName:documentName
    self
        setText:text
        plain:false
        anchor:anchor 
        documentName:documentName
        remember:false

    "Modified: 20.4.1996 / 19:42:52 / cg"
    "Created: 17.5.1996 / 17:11:07 / cg"
!

setText:text header:header footer:footer id:elementId anchor:anchor
    "set the text - see example"

    |doc s|

    document := nil.
    localImages := Dictionary new.

    printing ifFalse:[
        self scrollToTop.
    ].

    text isNil ifTrue:[
        document := nil
    ] ifFalse:[
        text isStream ifTrue:[
            s := text.
        ] ifFalse:[
            s := text asString readStream.
        ].
        document := HTMLParser new parseText:s withBindings:(self ampersandBindings).
    ].

    painter isNil ifTrue:[
        painter := HTMLDocumentPainter new.
    ].
    realized ifTrue:[
        printing ifTrue:[
            painter format:document for:formatContext resolver:self style:printStyle
        ] ifFalse:[
            painter format:document for:formatContext resolver:self style:style
        ]
    ].

    (printing not and:[shown]) ifTrue:[
        self redraw
    ]

    "Created: 17.5.1996 / 17:10:09 / cg"
    "Modified: 7.9.1996 / 15:46:30 / cg"
!

setText:text plain:plain anchor:anchor documentName:documentName remember:rem
    |t txt oldDocHeight oldDocWidth newDocHeight newDocWidth|

    printing ifFalse:[
        self rememberDocument:currentDocument text:nil action:currentAction inHistory:rem.
    ].

    plain ifTrue:[
        text isStream ifTrue:[
            txt := txt contents asString
        ] ifFalse:[
            txt := text
        ].
        t := (txt copy replChar:$< withString:'&lt;')
                replChar:$> withString:'&gt;'.
        t := '<PLAIN><CODE><PRE>' , Character cr asString , t
                , Character cr asString , '</PRE></CODE></PLAIN>'.
    ] ifFalse:[
        t := text
    ].

    oldDocHeight := self heightOfContents.
    oldDocWidth := self widthOfContents.

    self topView withWaitCursorDo:[
        currentURL := documentName.

        self setText:t anchor:anchor.

        documentName ~= currentDocument ifTrue:[
"/            documentName ~~ nil ifTrue:[
                currentDocument := documentName.
                currentAction := nil.
                self getVerticalPosition
"/            ]
        ].

        "/ Transcript showCR:'currentDir = ' , currentDirectoryName.
        "/ Transcript showCR:'current = ' , currentDocument.
        "/ Transcript showCR:'on ' , anchor printString.

        newDocHeight := self heightOfContents.
        newDocWidth := self widthOfContents.

        printing ifFalse:[
            (newDocHeight ~~ oldDocHeight 
            or:[newDocWidth ~~ oldDocWidth]) ifTrue:[
                self changed:#sizeOfContents
            ]
        ]
    ].

    "Created: 17.5.1996 / 17:11:44 / cg"
    "Modified: 7.9.1996 / 15:45:06 / cg"
!

setTopDirectoryName:aDirectoryName
    topDirectoryName := currentDirectoryName := aDirectoryName

    "Created: 20.5.1996 / 19:19:00 / cg"
!

showDocument:anchorHref text:anchorText action:action remember:rem source:source
    |rest localAnchor method file idx text found|

    "/    Transcript show:'anchor selected text='; showCR:anchorText.
    "/    Transcript show:'anchor selected href='; showCR:anchorHref.

    anchorHref notNil ifTrue:[
        method := 'file'.

        idx := anchorHref indexOf:$:.
        idx ~~ 0 ifTrue:[
            method := (anchorHref copyTo:(idx - 1)) asLowercase.
            rest := anchorHref copyFrom:(idx + 1).
        ] ifFalse:[
            rest := anchorHref
        ].

        "/
        "/ rest is everything after initial <method>:
        "/
        idx := rest indexOf:$# startingAt:idx + 1.
        found := false.
        [found or:[idx == 0]] whileFalse:[
            (rest at:idx+1) ~~ $# ifTrue:[
                found := true.
            ] ifFalse:[
                idx := rest indexOf:$# startingAt:idx+2
            ]
        ].
        idx ~~ 0 ifTrue:[
            file := rest copyTo:(idx - 1).
            file isEmpty ifTrue:[
                file := nil
            ].
            localAnchor := rest copyFrom:(idx + 1).
            localAnchor isEmpty ifTrue:[
                localAnchor := nil
            ].
        ] ifFalse:[
            file := rest
        ].

        action notNil ifTrue:[
            idx := action indexOf:$:.
            idx ~~ 0 ifTrue:[
                method := (action copyTo:(idx - 1)) asLowercase.
                rest := action copyFrom:(idx + 1).
            ] ifFalse:[
                method := 'doit'.
                rest := action
            ].

            self topView withExecuteCursorDo:[
                method = 'doit' ifTrue:[
                    Object errorSignal handle:[:ex |
                        'HTML: error while evaluating expression' infoPrintCR
                    ] do:[
                        Compiler evaluate:rest.
                    ].
                ] ifFalse:[
                    (method = 'plain'
                    or:[method = 'plaintext']) ifTrue:[
                        Object errorSignal handle:[:ex |
                            'HTML: error while evaluating expression' infoPrintCR
                        ] do:[
                            text := Compiler evaluate:rest.
                        ].
                        (text notNil and:[text notEmpty]) ifTrue:[
                            self setText:text plain:true anchor:nil documentName:anchorHref remember:rem.
                            visitedActions isNil ifTrue:[
                                visitedActions := Set new.
                            ].
                            visitedActions add:action.
                            currentAction := action.
                        ]
                    ] ifFalse:[
                        (method = 'html'
                        or:[method = 'htmltext']) ifTrue:[
                            Object errorSignal handle:[:ex |
                                'HTML: error while evaluating expression' infoPrintCR
                            ] do:[
                                text := Compiler evaluate:rest.
                            ].
                            (text notNil and:[text notEmpty]) ifTrue:[
                                self setText:text plain:source anchor:nil documentName:anchorHref remember:rem.
                                visitedActions isNil ifTrue:[
                                    visitedActions := Set new.
                                ].
                                visitedActions add:action.
                                currentAction := action.
                            ]
                        ] ifFalse:[
                            ('HTML: unsupported anchor method: ' , method) infoPrintCR
                        ]
                    ]
                ]
            ]
        ] ifFalse:[
            file notNil ifTrue:[
                method = 'file' ifTrue:[
                    self showFileDocument:file anchor:nil remember:rem source:source.
                ] ifFalse:[
                    (method = 'man') ifTrue:[
                        self withWaitCursorDo:[
                            text := HTMLDocGenerator
                                            manPageFor:rest.
                        ].
                        (text notNil and:[text notEmpty]) ifTrue:[
                            self setText:text plain:source anchor:nil documentName:anchorHref remember:rem.
                            visitedActions isNil ifTrue:[
                                visitedActions := Set new.
                            ].
                            visitedActions add:action.
                            currentAction := action.
                        ]
                    ] ifFalse:[
                        ('HTML: unsupported anchor method: ' , method) infoPrintCR
                    ]
                ]
            ]
        ].

        localAnchor notNil ifTrue:[
            "/ Transcript showCR:'local: ' , localAnchor.
            "/ Transcript show:'id='; showCR:(self anchorToId:localAnchor).
            self positionOnAnchor:localAnchor.
        ].
        currentLocalAnchor := localAnchor
    ]

    "Created: 17.5.1996 / 18:31:54 / cg"
    "Modified: 9.9.1996 / 17:52:00 / cg"
!

showFileDocument:urlString
    self showFileDocument:urlString anchor:nil remember:false

    "Created: 20.5.1996 / 19:19:45 / cg"
!

showFileDocument:urlString anchor:anchor remember:rem
    self showFileDocument:urlString anchor:anchor remember:rem source:false

    "Modified: 18.3.1996 / 13:16:46 / cg"
    "Created: 17.5.1996 / 17:07:55 / cg"
!

showFileDocument:urlString anchor:anchor remember:rem source:showSource
    |"fileName" s text newDoc
     oldDocHeight newDocHeight
     oldDocWidth newDocWidth myTopView doRemember
     newDirectoryName newDocumentName
     suffix info|

    doRemember := rem.
"/    (rem not and:[showSource]) ifTrue:[
"/        (hRefHistory isNil 
"/        or:[(hRefHistory findFirst:[:info | (info at:1) = currentDocument]) == 0])
"/        ifTrue:[
"/            doRemember := true
"/        ]
"/    ].

    self rememberDocument:currentDocument text:nil action:currentAction inHistory:doRemember.

    topDirectoryName isNil ifTrue:[
        topDirectoryName := '.'
    ].
    currentDirectoryName isNil ifTrue:[
        currentDirectoryName := '.'
    ].

    urlString notNil ifTrue:[
        (myTopView := self topView) withCursor:(Cursor read) do:[
            info := self documentStreamForURL:urlString.
            info notNil ifTrue:[
                s := info at:1.
                newDocumentName := info at:2.
                newDirectoryName := info at:3.
            ].

"/            fileName := self getDocumentFilename:urlString.
"/            fileName isDirectory ifTrue:[
"/                s := self pseudoDocumentStreamForDirectory:fileName pathName.
"/                s notNil ifTrue:[
"/                    newDirectoryName := urlString.
"/                    newDocumentName := fileName asFilename pathName.
"/                ]
"/            ] ifFalse:[
"/                ((suffix := fileName suffix) = 'html'
"/                or:[suffix = 'HTML'
"/                or:[suffix = 'htm'
"/                or:[suffix = 'HTM']]]) ifFalse:[
"/                    (suffix = '1') ifTrue:[
"/                        s := self pseudoDocumentStreamForManPageFile:fileName pathName.
"/                    ] ifFalse:[    
"/                        (suffix = 'gif'
"/                        or:[suffix = 'tiff'
"/                        or:[suffix = 'xbm'
"/                        or:[suffix = 'xpm']]]) ifTrue:[
"/                            s := self pseudoDocumentStreamForImageFile:fileName pathName.
"/                        ] ifFalse:[
"/                            s := self pseudoDocumentStreamForPlainFile:fileName pathName.
"/                        ]
"/                    ].    
"/                    s notNil ifTrue:[
"/                        newDirectoryName := urlString.
"/                        newDocumentName := fileName asFilename pathName.
"/                    ]
"/                ] ifTrue:[
"/                    s := fileName readStream.
"/                    s notNil ifTrue:[
"/                        newDirectoryName := s pathName asFilename directoryName.
"/                        newDocumentName := s pathName asFilename pathName.    
"/                    ]
"/                ]
"/            ].
"/
"/            uriHolder notNil ifTrue:[
"/                uriHolder value:fileName pathName.
"/            ].

            uriHolder notNil ifTrue:[
                uriHolder value:newDocumentName.
            ].

            s notNil ifTrue:[
                "/ text := s contents asString.
            ] ifFalse:[
"/                self showNotFound:fileName pathName.
                self showNotFound:urlString.
                ^ self.
            ].
        ].
    ].

    self setText:nil.

    (text isNil 
    and:[s isNil]) ifTrue:[
"/        fileName notNil ifTrue:[
"/            self showEmpty:fileName pathName.
"/        ].
        newDocumentName notNil ifTrue:[
            self showEmpty:newDocumentName
        ].
        ^ self
    ].

    showSource ifTrue:[
        text isNil ifTrue:[
            text := s contents asString.
            s close
        ].
        text := (text copy replChar:$< withString:'&lt;')
                    replChar:$> withString:'&gt;'.
        text := '<TITLE>Source of ' , newDocumentName , '</TITLE>'
                , Character cr asString 
                , '<PLAIN><CODE><PRE>' , Character cr asString , text
                , Character cr asString 
                , '</PRE></CODE></PLAIN>'.
    ].

    oldDocHeight := self heightOfContents.
    oldDocWidth := self widthOfContents.

    "/ have to change the current dir before setting the new document
    "/ otherwise, resolveImage requests fail

    currentDirectoryName := newDirectoryName.

    "
     need the full pathName ...
    "
    currentDirectoryName := currentDirectoryName asFilename pathName.

    text isNil ifTrue:[
        self 
            setText:s 
            anchor:anchor 
            documentName:newDocumentName.
    ] ifFalse:[
        self 
            setText:text 
            anchor:anchor 
            documentName:newDocumentName.
    ].

    s close.

    "Created: 17.5.1996 / 17:08:31 / cg"
    "Modified: 18.6.1996 / 10:35:40 / stefan"
    "Modified: 26.7.1996 / 19:28:14 / cg"
!

showFileDocument:urlString remember:rem
    self showFileDocument:urlString anchor:nil remember:rem

    "Created: 20.5.1996 / 19:27:34 / cg"
!

uriHolder:aValueHolder
    uriHolder := aValueHolder.
    aValueHolder notNil ifTrue:[
        self enableEvent:#pointerMotion
    ]

    "Created: 25.7.1996 / 12:04:57 / cg"
    "Modified: 25.7.1996 / 12:20:09 / cg"
! !

!HTMLDocumentView methodsFor:'button actions'!

menu_back
    |fileName historyEntry action anchor nm pos text|

    (hRefHistory isNil or:[hRefHistory isEmpty]) ifTrue:[
        ^ self
    ].

    historyEntry := hRefHistory removeLast.
"/ Transcript showCR:historyEntry printString.

    fileName := historyEntry at:1.
    action := historyEntry at:2.
    anchor := historyEntry at:3.
    pos := historyEntry at:4 ifAbsent:nil.
    text := historyEntry at:5 ifAbsent:nil.

"/Transcript showCR:'back to ' , fileName.
"/Transcript showCR:'anchor ' , anchor printString.

    text notNil ifTrue:[
        self 
            setText:text 
            anchor:anchor 
            documentName:nil.
    ] ifFalse:[
        anchor notNil ifTrue:[
            nm := fileName , '#' , anchor.
        ] ifFalse:[
            nm := fileName
        ].
        self 
            showDocument:fileName 
            text:nil 
            action:action 
            remember:false
            source:false.
    ].

    anchor notNil ifTrue:[
        self positionOnAnchor:anchor
    ] ifFalse:[
        pos notNil ifTrue:[
            self scrollVerticalToPercent:pos.
        ]
    ]

    "Modified: 26.4.1996 / 17:40:27 / cg"
    "Created: 20.5.1996 / 20:30:25 / cg"
!

menu_help 
    currentDirectoryName := topDirectoryName.
    self showFileDocument:(self helpDocumentName) remember:true.

    "Modified: 20.4.1996 / 22:05:44 / cg"
    "Created: 20.5.1996 / 20:30:25 / cg"
!

menu_home
    currentDirectoryName := topDirectoryName.
    homeDocument isNil ifTrue:[
        homeText isNil ifTrue:[
            ^ self
        ].
        self 
            setText:homeText 
            anchor:nil 
            documentName:nil.
    ] ifFalse:[
        self showFileDocument:homeDocument remember:false
    ].

    "Modified: 26.4.1996 / 17:35:45 / cg"
    "Created: 20.5.1996 / 20:30:25 / cg"
!

menu_print 
    |dir text method i|

    currentAction notNil ifTrue:[
        (i := currentAction indexOf:$:) == 0 ifTrue:[
            ^ self
        ].
        method := (currentAction copyTo:(i-1)) asLowercase.

        (method = 'html' or:[method = 'htmltext']) ifFalse:[
            ^ self
        ].
        self topView withExecuteCursorDo:[
            Object errorSignal handle:[:ex |
                'HTML: error while evaluating expression' infoPrintCR
            ] do:[
                text := Compiler evaluate:(currentAction copyFrom:(i+1)).
            ]
        ].
        (text notNil and:[text notEmpty]) ifFalse:[
            ^ self
        ].
        dir := currentDirectoryName
    ] ifFalse:[
        dir := currentDocument
    ].

    self topView withWaitCursorDo:[
        Processor activeProcess withLowerPriorityDo:[
            HTMLDocumentPrinter print:dir contents:text bindings:(self ampersandBindings) style:printStyle.
        ]
    ].


!

menu_printPage 
    |psText printStream how|

    Printer isNil ifTrue:[
        ^ self warn:'no printer.'.
    ].

    how := 2.  "/ postscript
    Printer supportsPostscript ifFalse:[
        self warn:'no postscript printer - ugly printing.'.
        how := 1.   "/ indented text
    ].

    self topView withWaitCursorDo:[
        psText := self getPageTextPretty:how.

        printStream := Printer newNative.
        printStream nextPutAllUntranslated:psText.
        printStream basicClose.
    ]

    "Created: 20.5.1996 / 20:30:25 / cg"
    "Modified: 5.9.1996 / 18:09:26 / cg"
!

menu_quit
    self topView destroy

    "Created: 20.5.1996 / 20:30:25 / cg"
!

menu_reload 
    |where txt|

    (currentDocument isNil 
    and:[currentAction isNil
    and:[homeText isNil]]) ifTrue:[
        ^ self
    ].

    printing ifFalse:[
        self getVerticalPosition.
        where := currentPercentage.
    ].
    currentDocument isNil ifTrue:[
        self setText:homeText
    ] ifFalse:[
        self 
            showDocument:currentDocument
            text:txt 
            action:currentAction 
            remember:false
            source:false.
    ].
    printing ifFalse:[
        self scrollVerticalToPercent:where.
    ].

    "Created: 20.5.1996 / 20:30:25 / cg"
    "Modified: 7.9.1996 / 15:44:27 / cg"
!

menu_settings
    |box component examplesEnabled showingImages 
     fontSelection fontSizes tinyFont smallFont mediumFont largeFont group
     colors clrs s
     exampleColorHolder anchorColorHolder visitedAnchorColorHolder
     textColorHolder fontSize headerLines headerLinesWhenPrinting
     printTOC initialHeaderNumbers needReload newColor|

    examplesEnabled := enableExecution asValue.
    showingImages := showImages asValue.

    headerLines := (style at:#headerNumbers ifAbsent:false) asValue.
    headerLinesWhenPrinting := (printStyle at:#headerNumbers ifAbsent:false) asValue.
    printTOC := (printStyle at:#printTOC ifAbsent:false) asValue.
    initialHeaderNumbers := (printStyle at:#initialHeaderNumbers ifAbsent:#(1 1 1 1 1 1)).
    initialHeaderNumbers := initialHeaderNumbers inject:'' into:[:sSoFar :el | sSoFar , el printString , ' '].
    initialHeaderNumbers := initialHeaderNumbers withoutSeparators asValue.

    fontSizes := #(8 10 12 16 24).
    fontSize := style at:#fontSize.
    fontSelection := fontSizes indexOf:fontSize ifAbsent:1.
    fontSelection := fontSelection asValue.

    box := DialogBox new.
    box label:'Viewer settings'.
    box extent:200@300.

    box addCheckBox:(resources string:'Allow execution of examples') on:examplesEnabled.
    box addHorizontalLine.
    box addCheckBox:(resources string:'Show images') on:showingImages.

    box addHorizontalLine.
    box addComponent:(Button 
                        label:(resources string:'Flush cached images now') 
                        action:[self class flushCachedImages]).

    box addHorizontalLine.
    group := RadioButtonGroup new.
    group add:(box addCheckBox:(resources string:'tiny font') on:tinyFont) toggleView.
    group add:(box addCheckBox:(resources string:'small font') on:smallFont) toggleView.
    group add:(box addCheckBox:(resources string:'medium font') on:mediumFont) toggleView.
    group add:(box addCheckBox:(resources string:'large font') on:largeFont) toggleView.
    group add:(box addCheckBox:(resources string:'huge font') on:largeFont) toggleView.
    group valueChannel:fontSelection.

    box addHorizontalLine.
    box addCheckBox:(resources string:'Header numbering') on:headerLines.
    box addCheckBox:(resources string:'Header numbering when printing') on:headerLinesWhenPrinting.
    box addCheckBox:(resources string:'Print table of contents') on:printTOC.
    component := box 
                    addLabelledInputField:(resources string:'Numbering scheme:')
                    adjust:#right
                    on:initialHeaderNumbers 
                    tabable:true
                    separateAtX:0.6.
    component acceptOnLeave:false.
    box addHorizontalLine.

    colors := OrderedCollection new.
    colors add:('-').
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color blue)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color blue darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:0 blue:100))).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:0 blue:100) darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:0 blue:100) darkened darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color red)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color red darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color red darkened darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:100 blue:0))).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:100 blue:0) darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:100 green:100 blue:0) darkened darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color green)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color green darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color green darkened darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:0 green:100 blue:100))).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:0 green:100 blue:100) darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->(Color red:0 green:100 blue:100) darkened darkened)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color black)).
    colors add:('    ' asText emphasizeAllWith:(#backgroundColor->Color white)).

    s := '    ' asText emphasizeAllWith:(#backgroundColor->(style at:#anchorFGColor)).
    clrs := colors copy addFirst:s; yourself.
    anchorColorHolder := s asValue.
    box
        addLabelledField:((ComboListView on:anchorColorHolder) list:clrs)
        label:(resources string:'Anchor color:')
        adjust:#left 
        tabable:true 
        separateAtX:0.7.

    s := '    ' asText emphasizeAllWith:(#backgroundColor->(style at:#visitedAnchorFGColor)).
    clrs := colors copy addFirst:s; yourself.
    visitedAnchorColorHolder := s asValue.
    box
        addLabelledField:((ComboListView on:visitedAnchorColorHolder) list:clrs)
        label:(resources string:'Visited anchor color:')
        adjust:#left 
        tabable:true 
        separateAtX:0.7.

    s := '    ' asText emphasizeAllWith:(#backgroundColor->(style at:#exampleFGColor)).
    clrs := colors copy addFirst:s; yourself.
    exampleColorHolder := s asValue.
    box
        addLabelledField:((ComboListView on:exampleColorHolder) list:clrs)
        label:(resources string:'Clickable example color:')
        adjust:#left 
        tabable:true 
        separateAtX:0.7.

    s := '    ' asText emphasizeAllWith:(#backgroundColor->(style at:#textFGColor)).
    clrs := colors copy addFirst:s; yourself.
    textColorHolder := s asValue.
    box
        addLabelledField:((ComboListView on:textColorHolder) list:clrs)
        label:(resources string:'Text color:')
        adjust:#left 
        tabable:true 
        separateAtX:0.7.

    box addHelpButtonFor:'HTMLDocView/settings.html'.
    box addAbortButton; addOkButton.
    box showAtPointer.

    box accepted ifTrue:[
        needReload := false.

        anchorColorHolder value notNil ifTrue:[
            newColor := (anchorColorHolder value emphasisAt:1) value.
            newColor ~= (style at:#anchorFGColor) ifTrue:[
                style at:#anchorFGColor put:newColor.
                needReload := true.
            ]
        ].
        visitedAnchorColorHolder value notNil ifTrue:[
            newColor := (visitedAnchorColorHolder value emphasisAt:1) value.
            newColor ~= (style at:#visitedAnchorFGColor) ifTrue:[
                style at:#visitedAnchorFGColor put:newColor.
                needReload := true.
            ]
        ].
        exampleColorHolder value notNil ifTrue:[
            newColor := (exampleColorHolder value emphasisAt:1) value.
            newColor ~= (style at:#exampleFGColor) ifTrue:[
                style at:#exampleFGColor put:newColor.
                needReload := true.
            ]
        ].
        textColorHolder value notNil ifTrue:[
            newColor :=(textColorHolder value emphasisAt:1) value.
            newColor ~= (style at:#textFGColor) ifTrue:[
                style at:#textFGColor put:newColor.
                needReload := true.
            ]
        ].

        (style at:#headerNumbers) ~~ headerLines value ifTrue:[
            style at:#headerNumbers put:headerLines value.
            needReload := true.
        ].
        printStyle at:#headerNumbers put:headerLinesWhenPrinting value.
        printStyle at:#printTOC put:printTOC value.
        initialHeaderNumbers := initialHeaderNumbers value.
        initialHeaderNumbers := '#(' , initialHeaderNumbers , ')'.
        initialHeaderNumbers := Object readFrom:initialHeaderNumbers onError:nil.
        (initialHeaderNumbers isArray 
        and:[initialHeaderNumbers ~= (printStyle at:#initialHeaderNumbers)]) ifTrue:[
            printStyle at:#initialHeaderNumbers put:initialHeaderNumbers.
            style at:#initialHeaderNumbers put:initialHeaderNumbers.
            headerLines value ifTrue:[
                needReload := true.
            ]
        ].

        self enableExecution:examplesEnabled value.
        showImages := showingImages value.

        fontSize ~~ (fontSizes at:fontSelection value) ifTrue:[
            style at:#fontSize put: (fontSizes at:fontSelection value).
            self initFonts:style.
            self initFonts:printStyle.
            needReload := true.
        ].

        needReload ifTrue:[
            self menu_reload
        ]
    ]

    "Created: 20.5.1996 / 20:30:26 / cg"
    "Modified: 7.9.1996 / 19:37:55 / cg"
!

menu_source 
    |text|

    device ctrlDown ifTrue:[
        text := self
                    getDocumentSource:currentDocument 
                    action:currentAction.
        EditTextView openWith:text title:(self topView label , ' - source').
        ^ self
    ].

    self 
        showDocument:currentDocument 
        text:nil 
        action:currentAction 
        remember:false 
        source:true

    "Created: 20.5.1996 / 20:30:26 / cg"
    "Modified: 21.5.1996 / 10:09:19 / cg"
! !

!HTMLDocumentView methodsFor:'defaults'!

helpDocumentName
    '../../doc/online/english/misc/usingHelp.html' asFilename exists ifTrue:[
        ^ '../../doc/online/english/misc/usingHelp.html'
    ].
    ^ 'doc/online/english/misc/usingHelp.html'

    "Modified: 3.9.1996 / 20:53:23 / cg"
! !

!HTMLDocumentView methodsFor:'document painter callBacks'!

imageFor:urlString
    "returns an image or nil"

    ^ self imageFor:urlString now:false

"/    |file|
"/
"/    file := self getDocumentFilename:urlString.
"/
"/    (file notNil and:[file exists]) ifTrue:[
"/        ^ Image fromFile:(file pathName)
"/    ].
"/    ^ nil

    "Created: 5.9.1996 / 17:48:29 / cg"
    "Modified: 5.9.1996 / 17:51:59 / cg"
!

imageFor:urlString now:now
    "returns an image or nil. The argument now specifies, if the image
     should be converted to a device image right NOW or not.
     Usually, it is false (i.e. no interrest in a device image) during
     document scan, and true when the image is to be displayed."

    |fileName path image bytes reds greens blues
     refs infoRec bits srcIndex dstIndex usedPixels
     n "{Class: SmallInteger }"
     w "{Class: SmallInteger }"
     h "{Class: SmallInteger }" 
     idx cacheEntry dImage|

    showImages ifFalse:[
        ^ nil
    ].

    (image := localImages at:urlString ifAbsent:nil) notNil ifTrue:[
        now ifTrue:[
            dImage := image on:self device.
            dImage ~~ image ifTrue:[    
                localImages at:urlString put:dImage.
                ^ dImage
            ].
        ].
        ^ image.
    ].

    fileName := self getDocumentFilename:urlString.
    fileName exists ifFalse:[
        "/ if we are in an action, try homeDir as well
        "/ makes life so much easier.
        "/ Transcript showCR:'try home'.

        (urlString startsWith:'/') ifFalse:[
            topDirectoryName isNil ifTrue:[
                topDirectoryName := self class documentFileFor:''
            ].
            fileName := (topDirectoryName , '/' , urlString) asFilename.
        ] ifTrue:[
            fileName := self getDocumentFilename:(urlString copyFrom:2).
        ]
    ].
    path := fileName pathName.

    "/ Transcript show:'image resolve: ' , urlString.
    "/ Transcript showCR:' -> ' , path.

    CachedImages notNil ifTrue:[
        idx := CachedImages findFirst:[:entry | 
                                                entry key = path ifTrue:[
                                                    cacheEntry := entry.    
                                                    image := entry value.
                                                    true.
                                                ] ifFalse:[
                                                    false       
                                                ]
                                      ].
        idx ~~ 0 ifTrue:[
            CachedImages addLast:(CachedImages removeAtIndex:idx).
            now ifTrue:[
                dImage := image on:self device.
                dImage ~~ image ifTrue:[
                    cacheEntry value:dImage.
                    image := dImage.
                ]
            ].
        ].

        image notNil ifTrue:[
            "/ Transcript showCR:'cached image ' , urlString.
        ].
    ].

    image isNil ifTrue:[
        fileName exists ifTrue:[
            "/ Transcript showCR:'exists as ', path.
            image := Image fromFile:path.
            image isNil ifTrue:[
                ('HTMLDocView: nonexisting image: ', path) errorPrintNL.
                ^ nil
            ].

            now ifTrue:[
                image := image on:self device.
            ].

            CachedImages isNil ifTrue:[
                CachedImages := OrderedCollection new:30.
            ] ifFalse:[
                CachedImages size > 30 ifTrue:[
                    CachedImages removeFirst
                ]
            ].

            CachedImages addLast:(path->image).

            self startImageFlushProcess.

        ]
    ].

    image isNil ifTrue:[
        ('HTMLDocView: nonexisting image: ', path) errorPrintNL.
        ^ nil
    ].

    localImages at:urlString put:image.
    ^ image.

    "Created: 17.5.1996 / 16:59:32 / cg"
    "Modified: 5.9.1996 / 17:53:15 / cg"
!

submitForm:info
    |method action argValues argNames arguments parser parseTree missingArg|

    (SignalSet with:Object abortSignal 
               with:Object errorSignal) handle:[:ex |
        'HTML: error when evaluating the expression' infoPrintCR.
        ex return
    ] do:[
        self topView withExecuteCursorDo:[
             method := info at:#method.
             (method withoutSpaces asLowercase startsWith:'doit') ifTrue:[
                 action := method copyFrom:(method indexOf:$|) + 1.

                 action := '[' , action , ']'.

                 argValues := info at:#attributes.
                 "
                  ok, the action contains a block
                  and the attributes contains the arg <-> value associations
                  first, parse the block (suing the compiler) to
                  get more symbolic info about it ...
                 "
"/              Transcript showCR:action.

                 parser := Parser for:action in:nil.
                 parser nextToken.
                 parseTree := parser block.   

"/              Transcript showCR:parseTree.

                 (parseTree notNil and:[parseTree ~~ #Error]) ifTrue:[
                     parseTree isBlock ifTrue:[
                         parseTree arguments isNil ifTrue:[
                             arguments := #()
                         ] ifFalse:[
                             "
                              collect argument names
                             "
                             argNames := parseTree arguments collect:[:v | v name].

                             "
                              construct an array where the values are  ...
                             "
                             missingArg := false.
                             arguments := argNames collect:[:nm | |val|
                                                                     val := argValues at:nm asSymbol ifAbsent:nil.
                                                                     val isNil ifTrue:[missingArg := true].
                                                                     val].

                             "
                              any argument empty ?
                             "
                             missingArg ifTrue:[
                                 self warn:'you must fill out all entry fields first'.
                                 ^ self
                             ].

                             arguments := arguments asArray.
                         ].

"/                       Transcript showCR:arguments displayString.

                         enableExecution ifFalse:[
                             self warn:('execution disabled.\If it wasn''t, the code executed would be:\\',
                                        parseTree printString) withCRs.
                             ^ self
                         ].

                         "
                          now, call the block, catching errors
                         "
                         Object abortSignal handle:[:ex |
                             ex return
                         ] do:[
                             Object errorSignal handle:[:ex |
                                 Dialog warn:('an error occurred while performing the action.\'
                                              , '(' , ex errorString , ')\\Check the document\\' 
                                              , parseTree printString) withCRs.
                                 ex return.
                             ] do:[
                                |realBlock|

                                realBlock := Compiler evaluate:action.
                                realBlock isBlock ifTrue:[
                                    realBlock valueWithArguments:arguments.
                                ] ifFalse:[
                                    parseTree valueWithArguments:arguments.
                                ]
                             ]
                         ].
                         ^ self
                     ]
                 ].
                 self warn:('there is some error in the definition of the action\\Check the document\\' 
                            , parseTree printString) withCRs.
             ] ifFalse:[
                 self warn:'I will only execute special Smalltalk DOIT forms'.
             ]
         ]
    ]

    "Created: 21.5.1996 / 19:03:30 / cg"
    "Modified: 30.6.1996 / 16:02:46 / cg"
!

title:aString
    self topView label:aString.
    (aString includes:$-) ifTrue:[
        self topView iconLabel:(aString copyFrom:(aString indexOf:$-)+1) withoutSeparators
    ] ifFalse:[
        self topView iconLabel:'Documentation'
    ].

    "Created: 17.5.1996 / 17:47:36 / cg"
    "Modified: 21.5.1996 / 11:48:56 / cg"
!

visitTest:urlString action:actionString
    |dirName fullName fileName idx|

    actionString notNil ifTrue:[
        visitedActions isNil ifTrue:[^ false].
        ^ visitedActions includes:actionString.
    ].

    visitedUrls isNil ifTrue:[^ false].

"/ ('check visited: ' , urlString) printCR.

"/ Transcript showCR:'check visited: ' , urlString.

    idx := urlString indexOf:$#.
    idx ~~ 0 ifTrue:[
        fileName := urlString copyTo:(idx - 1).
    ] ifFalse:[
        fileName := urlString
    ].
    (fileName startsWith:'/') ifFalse:[
        fileName := (currentDirectoryName asFilename construct:fileName) pathName.
    ].
    ^ visitedUrls includes:fileName.

    "Created: 20.5.1996 / 14:09:51 / cg"
    "Modified: 8.7.1996 / 20:57:11 / cg"
! !

!HTMLDocumentView methodsFor:'event handling'!

anchorSelected:anchorHref text:anchorText action:anchorAction
    self 
        showDocument:anchorHref 
        text:anchorText
        action:anchorAction
        remember:true
        source:false

    "Modified: 22.4.1996 / 23:21:38 / cg"
    "Created: 17.5.1996 / 18:31:05 / cg"
!

buttonMotion:state x:x y:y
    |a|

    painter isNil ifTrue:[self halt].
    infoHolder isNil ifTrue:[^ self].

    a := painter anchorAt:x@y.
    a notNil ifTrue:[
        infoHolder value:(a hrefString)
    ] ifFalse:[
        infoHolder value:''
    ].

    "Created: 25.7.1996 / 12:19:07 / cg"
!

buttonPress:button x:x y:y
    |a|

    painter isNil ifTrue:[self halt].

    a := painter anchorAt:x@y.
    a notNil ifTrue:[
        a isExample ifTrue:[
            self exampleSelected:(a hrefString)
                       text:(a anchorText)
                     action:(a actionString).
        ] ifFalse:[
            painter flushAnchorColors.

            self anchorSelected:(a hrefString)
                       text:(a anchorText)
                     action:(a actionString)
        ]
    ].

    "Created: 17.5.1996 / 17:37:53 / cg"
    "Modified: 21.5.1996 / 23:08:18 / cg"
!

exampleSelected:anchorHref text:anchorText action:actionText
    |top wg plug action box l tv|

"/    Transcript show:'example selected text='; showCR:anchorText.

    actionText isNil ifTrue:[
        action := anchorText
    ] ifFalse:[
        action := actionText
    ].

    enableExecution ifFalse:[
        box := DialogBox new.
        l := box addTextLabel:('code execution disabled.\if it wasn''t, the action would be:\\') withCRs.
        l adjust:#left.
        box font:(Font family:'courier' size:12).
        tv := HVScrollableView for:TextView.
        tv contents:action.
        tv preferredExtent:400@200.
        tv extent:400@200.
        box addComponent:tv.
        box addOkButton.
        box show.
"/        self warn:('code execution disabled.\if it wasn''t, the action would be:\\' , action) withCRs.
        ^ self
    ].

    top := self topView.
        top withExecuteCursorDo:[
            plug := Plug new.
            plug respondTo:#error:position:to:from:
                 with:[:msg :p1 :p2 :who | Dialog warn:('syntax error in the example:\\' , msg) withCRs].
            plug respondTo:#correctableError:position:to:from:
                 with:[:msg :p1 :p2 :who | Transcript showCR:('warning in the example:\\' , msg) withCRs. false].
            plug respondTo:#warning:position:to:from:
                 with:[:msg :p1 :p2 :who | Transcript showCR:('warning in the example:\\' , msg) withCRs. false].

            AbortSignal handle:[:ex |
                ex return
            ] do:[
                Object errorSignal handle:[:ex |
                    Dialog warn:'error in the example:

' , ex errorString.
                    ex return
                ] do:[
                    Compiler 
                        evaluate:action
                        notifying:plug
                ]
            ]
        ]

    "Created: 20.5.1996 / 14:45:04 / cg"
    "Modified: 22.8.1996 / 13:41:26 / cg"
!

redraw
    self clear. self paint:Color black.
    document notNil ifTrue:[
        self redrawX:0 y:0 width:width height:height
    ]

    "Modified: 17.5.1996 / 17:33:49 / cg"
    "Created: 17.5.1996 / 17:35:07 / cg"
!

redrawX:x y:y width:w height:h
    document notNil ifTrue:[
        self clearRectangleX:x y:y width:w height:h.

        painter isNil ifTrue:[self halt].
        painter displayX:x y:y width:w height:h on:self.
    ]

    "Created: 17.5.1996 / 16:02:37 / cg"
    "Modified: 21.5.1996 / 18:12:18 / cg"
!

sizeChanged:how
    painter notNil ifTrue:[
        realized ifFalse:[
            "/ first resize
            painter format:document for:self resolver:self style:style.
        ] ifTrue:[
            painter reformatDocument.
        ]
    ].
    super sizeChanged:how.

    "Created: 20.5.1996 / 12:47:32 / cg"
    "Modified: 5.9.1996 / 21:20:55 / cg"
! !

!HTMLDocumentView methodsFor:'initialize - release'!

destroy
    painter release.

    OpenHTMLWidgets notNil ifTrue:[
        OpenHTMLWidgets removeValue:self ifAbsent:nil.
    ].
    OpenHelpViews notNil ifTrue:[
        OpenHelpViews removeValue:self ifAbsent:nil.
    ].
    OpenHTMLWidgets size == 0 ifTrue:[
       CachedImages := nil
    ].
    document := painter := homeText := localImages := currentDocument := nil.

    super destroy.

    "Modified: 15.2.1996 / 12:45:54 / stefan"
    "Created: 20.5.1996 / 20:35:31 / cg"
    "Modified: 17.6.1996 / 20:23:44 / cg"
!

initFonts:aStyle
    |fontSize|

    fontSize := aStyle at:#fontSize.

    aStyle at:#normalFont           put:(Font family:'times' face:'medium' style:'roman' size:fontSize).
    aStyle at:#boldNormalFont       put:(Font family:'times' face:'bold'   style:'roman' size:fontSize).
    aStyle at:#italicNormalFont     put:(Font family:'times' face:'medium' style:'italic' size:fontSize).
    aStyle at:#boldItalicNormalFont put:(Font family:'times' face:'bold'   style:'italic' size:fontSize).

    aStyle at:#codeFont             put:(Font family:'courier' face:'medium' style:'roman' size:fontSize).
    aStyle at:#boldCodeFont         put:(Font family:'courier' face:'bold'   style:'roman' size:fontSize).
    aStyle at:#italicCodeFont       put:(Font family:'courier' face:'medium' style:'oblique' size:fontSize).
    aStyle at:#boldItalicCodeFont   put:(Font family:'courier' face:'bold'   style:'oblique' size:fontSize).

    aStyle at:#normalSmallFont      put:(Font family:'times' face:'medium' style:'roman' size:fontSize-2).

    aStyle at:#h1Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.8) rounded).
    aStyle at:#h2Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.4) rounded).
    aStyle at:#h3Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 1.2) rounded).
    aStyle at:#h4Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize)).
    aStyle at:#h5Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 0.8) rounded).
    aStyle at:#h6Font               put:(Font family:'times' face:'bold' style:'roman' size:(fontSize * 0.8) rounded).

    "Modified: 19.7.1996 / 15:42:51 / cg"
    "Created: 5.9.1996 / 21:15:00 / cg"
!

initStyle
    <resource: #style (#htmlActiveAnchorForegroundColor
                       #htmlActiveAnchorBackgroundColor
                       #htmlAnchorForegroundColor
                       #htmlExampleAnchorForegroundColor
                       #htmlForegroundColor
                       #htmlVisitedAnchorForegroundColor
                       #htmlFontSize)>

    super initStyle.

    style := IdentityDictionary "HTMLStyle" new.
    style at:#activeAnchorFGColor  put:(styleSheet colorAt:'htmlActiveAnchorForegroundColor'  default:Color red).
    style at:#activeAnchorBGColor  put:(styleSheet colorAt:'htmlActiveAnchorBackgroundColor'  default:viewBackground).
    style at:#anchorFGColor        put:(styleSheet colorAt:'htmlAnchorForegroundColor'        default:Color blue).
    style at:#exampleFGColor       put:(styleSheet colorAt:'htmlExampleAnchorForegroundColor' default:(Color red:40 green:0 blue:0)).
    style at:#textFGColor          put:(styleSheet colorAt:'htmlForegroundColor'              default:Color black).
    style at:#visitedAnchorFGColor put:(styleSheet colorAt:'htmlVisitedAnchorForegroundColor' default:(Color red:15 green:2 blue:36)).
    style at:#fontSize             put:(styleSheet at:'htmlFontSize' default:10).
    style at:#headerNumbers        put:false.

    self initFonts:style.

    printStyle := IdentityDictionary "HTMLStyle" new.
    printStyle at:#activeAnchorFGColor  put:(style at:#anchorFGColor).
    printStyle at:#activeAnchorBGColor  put:(Color white).
    printStyle at:#anchorFGColor        put:(style at:#anchorFGColor).
    printStyle at:#exampleFGColor       put:(style at:#exampleFGColor).
    printStyle at:#textFGColor          put:(style at:#textFGColor).
    printStyle at:#visitedAnchorFGColor put:(style at:#anchorFGColor).
    printStyle at:#fontSize             put:10. "/ (style at:#fontSize).
    printStyle at:#headerNumbers        put:true.
    printStyle at:#printTOC             put:true.
    printStyle at:#initialHeaderNumbers put:#(1 1 1 1 1 1).
    style at:#initialHeaderNumbers put:#(1 1 1 1 1 1).

    Printer supportsColor ifFalse:[
        printStyle at:#activeAnchorFGColor  put:(Color black).
        printStyle at:#activeAnchorBGColor  put:(Color white).
        printStyle at:#anchorFGColor        put:(Color black).
        printStyle at:#exampleFGColor       put:(Color black).
        printStyle at:#textFGColor          put:(Color black).
        printStyle at:#visitedAnchorFGColor put:(Color black).
    ].

    self initFonts:printStyle.

    "Modified: 7.9.1996 / 15:25:04 / cg"
!

initialize
    |bg|

    super initialize.

    formatContext  := self.
    printing := false.
    viewBackground := self defaultViewBackgroundColor.

    (bg := viewBackground exactOn:device) isNil ifTrue:[
        "/ no - never use dithered colors for the background
        'HTML: try nearby exact background color' infoPrintCR.
        bg := viewBackground nearestOn:device
    ].
    bg colorId isNil ifTrue:[
        'HTML: cannot get undithered background color' infoPrintCR.
    ].

    showImages := true.
    topDirectoryName := currentDirectoryName := '.'.

    self paint:Color black.
    enableExecution := true.

    "Created: 17.5.1996 / 17:01:55 / cg"
    "Modified: 7.9.1996 / 15:46:48 / cg"
!

recreate
    super recreate.
    painter notNil ifTrue:[
        painter getFontsFor:self
    ]

    "Created: 15.6.1996 / 17:01:52 / cg"
! !

!HTMLDocumentView methodsFor:'internal pseudo documents'!

documentStreamForURL:urlString
    |s fileName path suffix directoryName documentName|

    fileName := self getDocumentFilename:urlString.
    path := fileName pathName.

    fileName isDirectory ifTrue:[
        s := self pseudoDocumentStreamForDirectory:path.
        s notNil ifTrue:[
            directoryName := urlString.
            documentName := path.
        ]
    ] ifFalse:[
        ((suffix := fileName suffix) = 'html'
        or:[suffix = 'HTML'
        or:[suffix = 'htm'
        or:[suffix = 'HTM']]]) ifTrue:[
            s := fileName readStream.
            s notNil ifTrue:[
                directoryName := s pathName asFilename directoryName.
                documentName := path.    
            ]
        ] ifFalse:[
            (suffix = '1') ifTrue:[
                s := self pseudoDocumentStreamForManPageFile:path.
            ] ifFalse:[    
                (suffix = 'gif'
                or:[suffix = 'tiff'
                or:[suffix = 'xbm'
                or:[suffix = 'xpm']]]) ifTrue:[
                    s := self pseudoDocumentStreamForImageFile:path.
                ] ifFalse:[
                    suffix = 'ps' ifTrue:[
                        s := self pseudoDocumentStreamForPostscriptFile:path.
                    ] ifFalse:[
                        s := self pseudoDocumentStreamForPlainFile:path.
                    ]
                ]
            ].    
            s notNil ifTrue:[
                directoryName := urlString.
                documentName := path.
            ]
        ]
    ].
    s notNil ifTrue:[
        ^ Array with:s with:documentName with:directoryName
    ].
    ^ nil

    "Modified: 9.9.1996 / 17:46:26 / cg"
!

pseudoDocumentStreamForDirectory:url
    "a directory in an HREF - create pseudo document"

    |s urlDir entry|

    s := WriteStream on:''.

    s nextPutAll:'<title>Directory listing of '.
    s nextPutAll:url.
    s nextPutAll:'</title>'; cr.

    s nextPutAll:'<h1>Directory listing of '.
    s nextPutAll:url.
    s nextPutAll:'</h1>'; cr.

    s nextPutAll:'<pre>'; cr.

    (url asFilename pathName ~= '/') ifTrue:[
        s nextPutAll:'<a HREF="'.
        s nextPutAll:url asFilename directoryName asString.
        s nextPutAll:'">Up to higher level directory</A><BR>'; cr.
    ].

    urlDir := url asFilename.
    urlDir directoryContents do:[:fileName |
        |f nm suffix info size n unitString sizeString fn|

        (fileName ~= '.'
        and:[fileName ~= '..'
        and:[(fileName startsWith:'.') not]]) ifTrue:[
            f := urlDir construct:fileName.

            s nextPutAll:'<a HREF="'.
            s nextPutAll:f pathName.
            s nextPutAll:'"><IMG border=0 SRC="'.
            f isDirectory ifTrue:[
                nm := 'internal-gopher-menu'.
            ] ifFalse:[
                suffix := f suffix.
                FileExtensionTable notNil ifTrue:[
                    entry := FileExtensionTable at:suffix ifAbsent:nil.
                    entry notNil ifTrue:[
                        nm := entry first
                    ]
                ].
                nm isNil ifTrue:[
                    nm := 'internal-gopher-unknown'
                ].
            ].

            fn := fileName contractTo:32.

            s nextPutAll:nm.
            s nextPutAll:'"> '.
            s nextPutAll:fn.
            s nextPutAll:(String new:32 - fn size).
            s nextPutAll:'</A>'.

            info := f info.
            info notNil ifTrue:[
                size := (info at:#size).
                unitString := ' bytes'.
                size < (500 * 1024) ifTrue:[
                    size < 1024 ifTrue:[
                        n := size
                    ] ifFalse:[
                        n := (size * 10 // 1024 / 10.0).
                        unitString := ' Kb   '
                    ]
                ] ifFalse:[
                    n := (size * 10 // 1024 // 1024 / 10.0).
                    unitString := ' Mb   '
                ].
                sizeString :=  (n printStringLeftPaddedTo:5) , unitString.
                s nextPutAll:sizeString; space.
                s nextPutAll:(((info at:#modified) asDate displayString) paddedTo:12).
                s space.
                s nextPutAll:(info at:#modified) asTime displayString.
            ].

            s cr
        ]
    ].
    s nextPutAll:'</pre>'.

    ^ ReadStream on:s contents

    "Created: 8.7.1996 / 19:00:00 / cg"
    "Modified: 10.7.1996 / 14:43:54 / cg"
!

pseudoDocumentStreamForImageFile:url
    "an image in an HREF - create pseudo document"

    |s text|

    s := WriteStream on:''.

    s nextPutAll:'<title>'.
    s nextPutAll:url.
    s nextPutAll:'</title>'; cr.

    s nextPutAll:'<IMG SRC="' , url , '" border=0>'; cr.
    s nextPutAll:'</pre>'.

    ^ ReadStream on:s contents

    "Created: 8.7.1996 / 20:28:26 / cg"
    "Modified: 8.7.1996 / 20:37:26 / cg"
!

pseudoDocumentStreamForManPageFile:url
    "a plainFile in an HREF - create pseudo document"

    |s text|

    s := WriteStream on:''.

    s nextPutAll:'<title>Man output of '.
    s nextPutAll:url.
    s nextPutAll:'</title>'; cr.


    text := (HTMLDocGenerator manPageFor:url
                              manCommand:'nroff -man ' , url).

    s nextPutAll:text.

    ^ ReadStream on:s contents

    "Modified: 9.9.1996 / 17:45:53 / cg"
    "Created: 9.9.1996 / 17:46:41 / cg"
!

pseudoDocumentStreamForPlainFile:url
    "a plainFile in an HREF - create pseudo document"

    |s nm text|

    s := WriteStream on:''.

    s nextPutAll:'<title>'.
    s nextPutAll:url.
    s nextPutAll:'</title>'; cr.

    s nextPutAll:'<pre>'; cr.
    nm := url asFilename.
    nm isReadable ifTrue:[
        text := nm contentsOfEntireFile.
    ] ifFalse:[
        text := 'File not readable'
    ].
    text := text replChar:$< withString:'&lt;'.
    text := text replChar:$> withString:'&gt;'.

    s nextPutAll:text.
    s nextPutAll:'</pre>'.

    ^ ReadStream on:s contents

    "Created: 8.7.1996 / 19:42:07 / cg"
    "Modified: 10.7.1996 / 14:36:58 / cg"
! !

!HTMLDocumentView methodsFor:'misc'!

ampersandBindings
    |bindings|

    bindings := Dictionary new.
    bindings at:#date        put:(Date today printString).
    bindings at:#url         put:currentURL.
    bindings at:#exception   put:'no exception information'.
    bindings at:#message     put:'no message'.
    ^ bindings

    "Modified: 10.7.1996 / 15:45:55 / cg"
!

getVerticalPosition
     |h|

     h := self heightOfContents.
     h == 0 ifTrue:[^ 0].
     currentPercentage := self yOriginOfContents / h * 100.

    "Created: 17.5.1996 / 17:12:44 / cg"
    "Modified: 21.5.1996 / 15:24:54 / cg"
!

positionOnAnchor:aLocalAnchor
    |pos|

    pos := painter positionOfAnchor:aLocalAnchor.
    pos isNil ifFalse:[
        self scrollVerticalTo:((pos y - 10) max:0).
        self getVerticalPosition.
    ].

"/    self halt

    "Created: 17.5.1996 / 18:37:04 / cg"
    "Modified: 20.5.1996 / 21:53:02 / cg"
!

showNotFound:aFileName
    self setText:'
<HTML>
<BODY>
<STRONG>
cannot find document:
</STRONG>
<P>
' , aFileName , '
</BODY>
</HTML>
'       anchor:nil.

    "Created: 22.5.1996 / 14:03:03 / cg"
!

startImageFlushProcess
    ImageFlushProcess notNil ifTrue:[
        ImageFlushProcess terminate "/ NoSignal
    ].
    ImageFlushProcess := [
                            Delay waitForSeconds:ImageFlushDelay.
                            ImageFlushProcess := nil.    
                            CachedImages := nil.
                         ] fork.

    "Created: 17.6.1996 / 17:56:09 / cg"
! !

!HTMLDocumentView methodsFor:'queries'!

heightForLines:numberOfLines
    "return the height of the receiver, if numberOfLines are to be displayed"

    |normalFont|

    "need a device font for query"
    normalFont := (style at:#normalFont) on:device.
    ^ numberOfLines * normalFont height + normalFont descent + (margin * 2)

    "Created: 6.9.1996 / 17:58:45 / stefan"
!

heightOfContents
    painter isNil ifTrue:[^ 0].

    ^ painter height

    "Created: 17.5.1996 / 17:29:15 / cg"
    "Modified: 17.5.1996 / 17:32:19 / cg"
!

preferredExtentForLines:numLines cols:numCols
    "get an approximation for the extent to display numLines and
     numCols in normal fontsize"

    |normalFont|

    "need a device font for query"
    normalFont := (style at:#normalFont) on:device.
    ^ (((normalFont widthOf:'x') * numCols + margin + margin) 
      @ 
      (normalFont height * numLines + margin + margin + normalFont descent)).

    "Created: 6.9.1996 / 17:59:30 / stefan"
!

widthOfContents
    painter isNil ifTrue:[^ self innerWidth].
    ^ painter width

    "Created: 17.5.1996 / 17:29:20 / cg"
    "Modified: 17.5.1996 / 17:36:50 / cg"
! !

!HTMLDocumentView methodsFor:'scrolling'!

XXcatchExpose
    painter hideAllWidgets.
"/    self handleExposeEvents.
    super catchExpose.

    "Modified: 24.5.1996 / 19:08:40 / cg"
    "Created: 24.5.1996 / 20:09:32 / cg"
!

XXwaitForExpose
    painter showWidgets.
    super waitForExpose.

    "Modified: 24.5.1996 / 15:52:42 / cg"
    "Created: 24.5.1996 / 20:09:38 / cg"
! !

!HTMLDocumentView  class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/HTMLDocumentView.st,v 1.59 1996-09-10 13:34:20 cg Exp $'
! !
HTMLDocumentView initialize!
