WriteStream subclass:#HTMLFilterStream
	instanceVariableNames:'runs outStream loHeaders'
	classVariableNames:''
	poolDictionaries:''
	category:'Streams'
!


!HTMLFilterStream  class methodsFor:'instance creation'!

on:aStream
    |newStream|

    newStream := super on:nil.
    newStream outStream:aStream.
    ^ newStream
! !

!HTMLFilterStream methodsFor:'elements - document'!

body
    outStream nextPutAll:'<BODY>'; cr.

!

bodyEnd
    outStream nextPutAll:'</BODY>'; cr.

!

head
    outStream nextPutAll:'<HEAD>'; cr.

!

headEnd
    outStream nextPutAll:'</HEAD>'; cr.

!

html
    outStream nextPutAll:'<HTML>'; cr.

!

htmlEnd
    outStream nextPutAll:'</HTML>'; cr.

!

title:aTitle
    outStream nextPutAll:'<TITLE>', aTitle asString, '</TITLE>'; cr.
! !

!HTMLFilterStream methodsFor:'elements - special'!

img:aFile hspace:hspace

    outStream nextPutAll:'<IMG src="'
                        , aFile asString
                        , '" hspace='
                        , hspace printString
                        , '>'; cr


! !

!HTMLFilterStream methodsFor:'elements - structure'!

br
    outStream nextPutAll:'<BR>'; cr.


!

h:number
    "begin header with number
    "
    outStream nextPutAll:'<H', number printString, '>'.
    loHeaders add:number.


!

hEnd
    "end header definition
    "
    |number|

    loHeaders notEmpty ifTrue:[
        number := loHeaders removeLast.
        outStream nextPutAll:'</H', number printString, '>'; cr
    ].


!

hr
    outStream nextPutAll:'<HR>'; cr.

!

p
    outStream nextPutAll:'<P>'; cr.

!

pEnd
    outStream nextPutAll:'</P>'; cr.

!

pre
    outStream nextPutAll:'<PRE>'; cr.

!

preEnd
    outStream nextPutAll:'</PRE>'; cr.

! !

!HTMLFilterStream methodsFor:'initialization'!

outStream:aStream
    outStream := aStream.
    loHeaders := OrderedCollection new.
    collection := nil.
! !

!HTMLFilterStream methodsFor:'private accessing'!

emphasis:attr on:on

    |id|

                attr == #bold      ifTrue:[ id := $B
    ] ifFalse:[ attr == #italic    ifTrue:[ id := $I
    ] ifFalse:[ attr == #underline ifTrue:[ id := $U
    ] ifFalse:[ attr == #strikeout ifTrue:[ id := $S
    ] ifFalse:[ ^ self
    ]]]].

                 outStream nextPut:$<.
    on ifFalse:[ outStream nextPut:$/ ].
                 outStream nextPut:id.
                 outStream nextPut:$>.







!

removeAllEmphasis

    runs notNil ifTrue:[
        runs do:[:attr| self emphasis:attr on:false ].
        runs := nil
    ].





! !

!HTMLFilterStream methodsFor:'writing'!

emphasis:emphasis

    |newRun|

    emphasis isNil ifTrue:[
        ^ self removeAllEmphasis
    ].

    runs isNil ifTrue:[
        runs := IdentitySet new.

        emphasis isSymbol ifTrue:[
            runs add:emphasis.
            self emphasis:emphasis on:true.
        ] ifFalse:[
            emphasis do:[:el|
                runs add:el.
                self emphasis:el on:true.
            ]
        ]
    ] ifFalse:[
        emphasis isSymbol ifTrue:[
            runs do:[:el|
                el == emphasis ifFalse:[
                    runs saveRemove:el.
                    self emphasis:el on:false
                ]
            ].
            runs rehash.

            (runs includes:emphasis) ifFalse:[
                runs add:emphasis.
                self emphasis:emphasis on:true.
            ]
        ] ifFalse:[

            newRun := IdentitySet new.

            runs do:[:el|
                (emphasis includes:el) ifFalse:[
                    self emphasis:el on:false
                ]
            ].

            emphasis do:[:el|
                (runs includes:el) ifFalse:[
                    self emphasis:el on:true
                ].
                newRun add:el.
            ].
            runs := newRun
        ]
    ]
!

nextPut:aCharacter
    "append the argument, anCharacter to the stream"

    aCharacter ~~ $< ifTrue:[ outStream nextPut:aCharacter ] 
                    ifFalse:[ outStream nextPutAll:'&lt;' ]
! !

!HTMLFilterStream  class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/HTMLFilterStream.st,v 1.1 1996-07-19 12:02:26 ca Exp $'
! !
